# -*- Mode: Python; py-indent-offset: 2 -*-
# -*- coding: utf-8 -*-
# vim: tabstop=2 shiftwidth=2 expandtab
# vim: set fileencoding=utf-8
#
# EXTRAITS DE LA LICENCE
#    Copyright CEA, contributeurs : Damien CALISTE, (2001-2010)
#
#    Adresse mèl :
#    CALISTE, damien P caliste AT cea P fr.
#
#    Ce logiciel est un programme informatique servant à visualiser des
#    structures atomiques dans un rendu pseudo-3D. 
#
#    Ce logiciel est régi par la licence CeCILL soumise au droit français et
#    respectant les principes de diffusion des logiciels libres. Vous pouvez
#    utiliser, modifier et/ou redistribuer ce programme sous les conditions
#    de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA 
#    sur le site "http://www.cecill.info".
#
#    Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
#    pris connaissance de la licence CeCILL, et que vous en avez accepté les
#    termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).


# LICENCE SUM UP
#    Copyright CEA, contributors: Damien CALISTE, (2001-2010)
#
#    E-mail address:
#    CALISTE, damien P caliste AT cea P fr.
#
#    This software is a computer program whose purpose is to visualize atomic
#    configurations in 3D.
#
#    This software is governed by the CeCILL  license under French law and
#    abiding by the rules of distribution of free software.  You can  use, 
#    modify and/ or redistribute the software under the terms of the CeCILL
#    license as circulated by CEA, CNRS and INRIA at the following URL
#    "http://www.cecill.info". 
#
#    The fact that you are presently reading this means that you have had
#    knowledge of the CeCILL license and that you accept its terms. You can
#    find a copy of this licence shipped with this software at
#    Documentation/licence.en.txt.

from ..module import get_introspection_module

v_sim = get_introspection_module('v_sim')

class VisuNodeArrayIter:
  def __init__(self, it):
    self.iter = it

  def __iter__(self):
    return self

  def next(self):
    if not(self.iter.next()):
      raise StopIteration
    return self.iter

def __NodeArrayIter__(it):
  return VisuNodeArrayIter(it)
setattr(v_sim.NodeArrayIter, "__iter__", __NodeArrayIter__)

class VisuNodeArrayIter2:
  def __init__(self, nodes):
    self.iter1 = nodes.iter_new()
    self.iter2 = nodes.iter_new()

  def __iter__(self):
    return self

  def next(self):
    if not(self.iter1.next2(self.iter2)):
      raise StopIteration
    return (self.iter1, self.iter2)

def newIt(nodes, kind = v_sim.NodeArrayIterType.NODES_BY_TYPE, elename = None):
  it = nodes.iter_new()
  it.type = kind
  if elename is not None:
    it.element = v_sim.Element.lookup(elename)
  return it
def __NodeArrayIter__(nodes):
  return VisuNodeArrayIter(newIt(nodes))
def __NodeArrayIterByNumber__(nodes):
  return VisuNodeArrayIter(newIt(nodes, v_sim.NodeArrayIterType.NODES_BY_NUMBER))
def __NodeArrayIterElements__(nodes):
  return VisuNodeArrayIter(newIt(nodes, v_sim.NodeArrayIterType.ELEMENTS))
def __NodeArrayIterVisible__(nodes):
  return VisuNodeArrayIter(newIt(nodes, v_sim.NodeArrayIterType.NODES_VISIBLE))
def __NodeArrayIterOriginal__(nodes):
  return VisuNodeArrayIter(newIt(nodes, v_sim.NodeArrayIterType.NODES_ORIGINAL))
def __NodeArrayIterForElement__(nodes, elename):
  return VisuNodeArrayIter(newIt(nodes, v_sim.NodeArrayIterType.NODES_FOR_ELEMENT, elename = elename))
def __NodeArrayIterByPairs__(nodes):
  return VisuNodeArrayIter2(nodes)


def __DataAllNodePositions__(data):
  nodes = []
  for it in data:
    nodes.append((data.getNodeCoordinates(it.node), \
                  it.node.number, it.element.getName()))
  return nodes

setattr(v_sim.NodeArray, "__iter__", __NodeArrayIter__)
setattr(v_sim.NodeArray, "__iterByNumber__", __NodeArrayIterByNumber__)
setattr(v_sim.NodeArray, "__iterElements__", __NodeArrayIterElements__)
setattr(v_sim.NodeArray, "__iterVisible__", __NodeArrayIterVisible__)
setattr(v_sim.NodeArray, "__iterOriginal__", __NodeArrayIterOriginal__)
setattr(v_sim.NodeArray, "__iterForElement__", __NodeArrayIterForElement__)
setattr(v_sim.NodeArray, "__iterByPairs__", __NodeArrayIterByPairs__)
setattr(v_sim.Data, "getAllNodePositions", __DataAllNodePositions__)

def __SceneShow__(scene, withToolbar = False):
  from gi.repository import GLib, Gtk

  try:
    v_sim.basic_initConfigFiles()
  except:
    pass
  
  r = v_sim.UiRenderingWindow.new_withGlScene(scene, withToolbar)

  w = Gtk.Window.new(Gtk.WindowType.TOPLEVEL)
  w.add(r)
  w.show_all()

  l = GLib.MainLoop.new(None, False)
  w.connect_object("destroy", GLib.MainLoop.quit, l)
  l.run()

setattr(v_sim.GlNodeScene, "show", __SceneShow__)

try:
  v_sim.basic_init()
  v_sim.plugins_init()
except:
  pass



#!/usr/bin/env python

import abinit
import numpy
import pygtk, gtk, gobject
import re, sys, os
import ElementTree
import v_sim

def delete_event(widget, event, data=None):
    return False

def destroy(widget, data=None):
    gtk.main_quit()

def loadData(widget, response_id, window):
  if (response_id != gtk.RESPONSE_ACCEPT):
    return
  if (widget is not window.get_data("fileSel")):
    widget.hide()

  # Remove old columns.
  view = window.get_data("treeview")
  for col in view.get_columns()[1:]:
    view.remove_column(col)

  wd = window.get_data("hboxResults")
  wd.set_sensitive(False)
  
  window.set_data("dtsets", None)
  v_sim.set(None)
  # We parse the file.
  try:
      toto = abinit.Dtsets(widget.get_filename())
  except Exception, err:
      window.get_data("fileSel").unselect_all()
      wd = gtk.MessageDialog(window, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, err.msg)
      wd.run()
      wd.destroy()
      return
  window.set_data("dtsets", toto)

  wd = window.get_data("hboxResults")
  wd.set_sensitive(True)
  
  # Create a new model.
  args = "gtk.ListStore(gobject.TYPE_STRING"
  for i in range(toto.get_ndtset()):
    args += ", gobject.TYPE_STRING, gobject.TYPE_STRING"
  args += ")"
  model = eval(args)
  modelFilter = model.filter_new()
  window.set_data("model", model)
  modelFilter.set_visible_func(hideCycleIter, window)
  modelSort = gtk.TreeModelSort(modelFilter)
  modelSort.set_sort_column_id(0, gtk.SORT_ASCENDING)
  view.set_model(modelSort)

  for strId in abinit.get_ids().iterkeys():
    iter = model.append()
    model.set(iter, 0, strId)
    default = toto.get(strId, 0)
    for i in range(toto.get_ndtset()):
        val = toto.get(strId, i + 1)
        if (numpy.all(val == default)):
            misc = "#505050"
        else:
            misc = "blue"
        if (val is None):
            val = "<span size=\"smaller\"><i>Unsupported type</i></span>"
        model.set(iter, 2 * i + 1, val.__str__(), 2 * i + 2, misc)
  # Add the column to the tree view.
  for i in range(1, toto.get_ndtset() + 1):
    col = view.insert_column_with_attributes(-1, "Value dtset %d" % i, \
                                             gtk.CellRendererText(), \
                                             markup = 2 * i - 1, foreground = 2 * i)
    col.set_resizable(True)
    col.set_sort_column_id(i)
  # Adapt the range for the spDtSets button.
  window.get_data("spDtSets").set_range(1, toto.get_ndtset() + 1)

def renderDtSet(widget, window):
    dtset = window.get_data("dtsets")
    iSet = int(window.get_data("spDtSets").get_value())
    znucl = dtset.get("znucl", iSet)
    pop = {}
    eles = []
    for i in znucl:
        ele = v_sim.element(int(i))
        pop[ele] = 0
        eles.append(ele)
    print pop
    typat = dtset.get("typat", iSet)
    for i in typat:
        pop[eles[i - 1]] += 1
    data = v_sim.data()
    data.setPopulation(pop)
    rprimd = dtset.get("rprimd_orig", iSet)
    coord = dtset.get("xred_orig", iSet)
    i = 0
    for xyz in coord:
        c = (rprimd[0][0] * xyz[0] + rprimd[0][1] * xyz[1] + rprimd[0][2] * xyz[2], \
             rprimd[1][0] * xyz[0] + rprimd[1][1] * xyz[1] + rprimd[1][2] * xyz[2], \
             rprimd[2][0] * xyz[0] + rprimd[2][1] * xyz[1] + rprimd[2][2] * xyz[2])
        data.addNode(eles[typat[i] - 1], c)
        i += 1
    box = rprimd.tolist()
    data.setBox((tuple(box[0]), tuple(box[1]), tuple(box[2])), True)
    v_sim.set(data)
               

def hideCycleIter(model, iter, window):
    (strId,) = model.get(iter, 0)
    if (strId is None):
      return False
    if (window.get_data("hideDefault").get_active()):
      dtsets = window.get_data("dtsets")
      hide = True
      for i in range(dtsets.get_ndtset()):
        (color,) = model.get(iter, 2 * i + 2)
        hide = hide and (color != "blue")
    else:
      hide = False
    return (not(hide) and re.match(window.get_data("filter"), strId) is not None)

def applyFilter(widget, window, entry):
    patStr = entry.get_text()
    if (patStr[-1] != "$"):
        patStr += "$"
    pat = re.compile(patStr)
    if (pat is not None):
        window.set_data("filter", pat)
        window.get_data("treeview").get_model().get_model().refilter()

def applyHide(widget, window):
  window.get_data("treeview").get_model().get_model().refilter()

def withArg(argv, window):
    fileSel = window.get_data("fileSel")
    fileSel.set_filename(os.path.abspath(sys.argv[1]))
    while (gtk.events_pending()):
        gtk.main_iteration()
    if (fileSel.get_filename() is not None):
        loadData(fileSel, gtk.RESPONSE_ACCEPT, window)
    else:
        raise ValueError("No file '%s'." % os.path.abspath(sys.argv[1]))
    return False

def XMLGetText(var):
  if (var is None):
    return ""

  if (var.text is not None):
    text = var.text.strip().replace("\n", " ") + " "
  else:
    text = ""
  for child in var.getchildren():
    if (child.tag == "p"):
        text += XMLGetText(child) + "\n"
    elif (child.tag == "ul"):
        text += "\n" + XMLGetText(child)
    elif (child.tag == "li"):
        text += "  * " + XMLGetText(child) + "\n"
    elif (child.tag == "br"):
        text += "\n"
    else:
        text += XMLGetText(child)
  if (var.tail is not None):
    text += var.tail.strip().replace("\n", " ") + " "
  return re.sub("  *", " ", text)
  
def selectAtt(selection, window):
    (model, iter) = selection.get_selected()
    if (iter is None):
        return
    iSets = []
    (att,) = model.get(iter, 0)
    window.get_data("lblKeyword").set_markup("<b>" + att + "</b>")
    window.get_data("lblMnemonic").set_markup("")
    window.get_data("lblCategory").set_markup("")
    window.get_data("lblType").set_markup("")
    window.get_data("lblDefault").set_markup("")
    window.get_data("description").set_text("")
    varlist = window.get_data("varlist")
    for var in varlist:
      if (var.find("keyword").text == att):
        window.get_data("lblMnemonic").set_markup(XMLGetText(var.find("mnemonics")))
        window.get_data("lblCategory").set_markup(XMLGetText(var.find("categories")))
        window.get_data("lblType").set_markup(XMLGetText(var.find("type")))
        window.get_data("lblDefault").set_markup(XMLGetText(var.find("default")))
        window.get_data("description").set_text(XMLGetText(var.find("description")))
        break

if __name__ == "__main__":
  window = gtk.Window(gtk.WINDOW_TOPLEVEL)
  window.set_border_width(5)
  window.connect("delete_event", delete_event)
  window.connect("destroy", destroy)

  vbox = gtk.VBox()
  window.add(vbox)

  hbox = gtk.HBox()
  vbox.pack_start(hbox, False, False, 5)

  wd = gtk.Label("<b>Analysing data file:</b>")
  wd.set_use_markup(True)
  wd.set_alignment(0., 0.5)
  wd.set_padding(10, 0)
  hbox.pack_start(wd, False, False, 0)

  wd = gtk.FileChooserDialog(title = "Choose an ABINIT data file", \
                             buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                                        gtk.STOCK_OPEN, gtk.RESPONSE_ACCEPT))
  wd.connect("response", loadData, window)
  fileSel = gtk.FileChooserButton(wd)
  window.set_data("fileSel", fileSel)
  hbox.pack_start(fileSel, True, True, 5)


  hbox = gtk.VPaned()
  hbox.set_sensitive(False)
  window.set_data("hboxResults", hbox)
  vbox.pack_start(hbox, True, True, 0)

  wd = gtk.VBox()
  hbox.pack1(wd, True, False)

  scroll = gtk.ScrolledWindow()
  scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
  scroll.set_shadow_type(gtk.SHADOW_ETCHED_OUT)
  wd.pack_start(scroll, True, True, 0)

  view = gtk.TreeView()
  wd.set_size_request(-1, 400)
  view.set_rules_hint(True)
  view.set_grid_lines(gtk.TREE_VIEW_GRID_LINES_VERTICAL)
  view.get_selection().set_mode(gtk.SELECTION_SINGLE)
  view.get_selection().connect("changed", selectAtt, window)
  window.set_data("treeview", view)
  scroll.add(view)

  col = view.insert_column_with_attributes(-1, "Attribute", \
                                           gtk.CellRendererText(), \
                                           markup=0)
  col.set_sort_column_id(0)

  hbox2 = gtk.HBox()
  wd.pack_start(hbox2, False, False, 0)
  entry = gtk.Entry()
  entry.set_text(".*")
  window.set_data("filter", re.compile(".*"))
  entry.connect("activate", applyFilter, window, entry)
  bt1 = gtk.Button(label="Filter")
  bt1.connect("clicked", applyFilter, window, entry)
  bt2 = gtk.ToggleButton(label="Hide defaults")
  bt2.connect("toggled", applyHide, window)
  window.set_data("hideDefault", bt2)
  hbox2.pack_end(bt2, False, False, 0)
  hbox2.pack_end(bt1, False, False, 0)
  hbox2.pack_end(entry, False, False, 0)


  pane = gtk.HPaned()
  hbox.pack2(pane, True, True)

  render = v_sim.main(width=400, height=400)
  pane.pack2(render, True, True)

  table = gtk.Table(6, 4)
  table.set_row_spacings(10);

  align = gtk.Alignment(xalign=1)
  table.attach(align, 0, 4, 5, 6, yoptions = gtk.SHRINK, xoptions = gtk.FILL | gtk.EXPAND, ypadding=5)
  wd = gtk.HBox()
  align.add(wd)
  lb = gtk.Label("Render dtset:")
  wd.pack_start(lb, False, False, 0)
  sp = gtk.SpinButton()
  window.set_data("spDtSets", sp)
  wd.pack_start(sp, False, False, 0)
  bt = gtk.Button(stock = gtk.STOCK_GO_FORWARD)
  bt.connect("clicked", renderDtSet, window)
  wd.pack_start(bt, False, False, 10)

  pane.pack1(table, True, False)


  wd = gtk.Label("<i>Keyword:</i>")
  wd.set_use_markup(True)
  wd.set_alignment(1., 0.5)
  table.attach(wd, 0, 1, 0, 1, xoptions = gtk.SHRINK | gtk.FILL, yoptions = gtk.SHRINK)
  wd = gtk.Label("")
  wd.set_alignment(0., 0.5)
  table.attach(wd, 1, 2, 0, 1, yoptions = gtk.SHRINK, xpadding = 5)
  window.set_data("lblKeyword", wd)

  wd = gtk.Label("<i>Mnemonic:</i>")
  wd.set_use_markup(True)
  wd.set_alignment(1., 0.5)
  table.attach(wd, 2, 3, 0, 1, xoptions = gtk.SHRINK | gtk.FILL, yoptions = gtk.SHRINK)
  wd = gtk.Label("")
  wd.set_line_wrap(True)
  wd.set_alignment(0., 0.5)
  table.attach(wd, 3, 4, 0, 1, yoptions = gtk.SHRINK, xpadding = 5)
  window.set_data("lblMnemonic", wd)

  wd = gtk.Label("<i>Found in:</i>")
  wd.set_use_markup(True)
  wd.set_alignment(1., 0.5)
  table.attach(wd, 0, 1, 1, 2, xoptions = gtk.SHRINK | gtk.FILL, yoptions = gtk.SHRINK)
  wd = gtk.Label("")
  wd.set_alignment(0., 0.5)
  table.attach(wd, 1, 2, 1, 2, yoptions = gtk.SHRINK, xpadding = 5)
  window.set_data("lblFound", wd)

  wd = gtk.Label("<i>Category:</i>")
  wd.set_use_markup(True)
  wd.set_alignment(1., 0.5)
  table.attach(wd, 2, 3, 1, 2, xoptions = gtk.SHRINK | gtk.FILL, yoptions = gtk.SHRINK)
  wd = gtk.Label("")
  wd.set_alignment(0., 0.5)
  table.attach(wd, 3, 4, 1, 2, yoptions = gtk.SHRINK, xpadding = 5)
  window.set_data("lblCategory", wd)

  wd = gtk.Label("<i>Type:</i>")
  wd.set_use_markup(True)
  wd.set_alignment(1., 0.5)
  table.attach(wd, 0, 1, 2, 3, xoptions = gtk.SHRINK | gtk.FILL, yoptions = gtk.SHRINK)
  wd = gtk.Label("")
  wd.set_line_wrap(True)
  wd.set_alignment(0., 0.5)
  table.attach(wd, 1, 2, 2, 3, yoptions = gtk.SHRINK, xpadding = 5)
  window.set_data("lblType", wd)

  wd = gtk.Label("<i>Default:</i>")
  wd.set_use_markup(True)
  wd.set_alignment(1., 0.5)
  table.attach(wd, 2, 3, 2, 3, xoptions = gtk.SHRINK | gtk.FILL, yoptions = gtk.SHRINK)
  wd = gtk.Label("")
  wd.set_line_wrap(True)
  wd.set_alignment(0., 0.5)
  table.attach(wd, 3, 4, 2, 3, yoptions = gtk.SHRINK, xpadding = 5)
  window.set_data("lblDefault", wd)

  wd = gtk.Label("<i>Description:</i>")
  wd.set_use_markup(True)
  wd.set_alignment(1., 0.5)
  table.attach(wd, 0, 1, 3, 4, gtk.SHRINK | gtk.FILL, yoptions = gtk.SHRINK)

  scroll = gtk.ScrolledWindow()
  scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
  scroll.set_shadow_type(gtk.SHADOW_ETCHED_OUT)
  table.attach(scroll, 0, 4, 4, 5, xpadding = 5)
  buf = gtk.TextBuffer()
  window.set_data("description", buf)
  wd = gtk.TextView(buf)
  wd.set_editable(False)
  wd.set_wrap_mode(gtk.WRAP_WORD)
  wd.set_size_request(400, -1)
  scroll.add(wd)

  window.show_all()

  if (len(sys.argv) > 1):
    gobject.idle_add(withArg, sys.argv, window)

  # We parse the variable defs.
  varlist = ElementTree.parse("varlists.xml").findall("list/variable")
  window.set_data("varlist", varlist)

  gtk.main()



#!/usr/bin/env python
from gi.repository import v_sim
import traceback

def parseCNT(format, filename, data, iset, cancel):
  print "test file: %s for VisuData: %s(%d)" % (filename, data, iset)
  valid = False
  try:
    # Read the given file.
    f = open(filename, "r")
    desc = f.readline()
    f.readline()
    box = map(float, f.readline().split()) + \
          map(float, f.readline().split()) + \
          map(float, f.readline().split())
    if len(box) != 9:
      raise ValueError
    elements = f.readline().split()
    nElements = map(int, f.readline().split())
    # From here, we consider that this file is indeed a VASP file.
    valid = True
    f.readline()
    f.readline()
    coords = []
    for line in f.xreadlines():
      coords.append(map(float, line.split()[:3]))
    f.close()
    allElements = []
    for (ele, n) in zip(elements, nElements):
      allElements += [ele, ] * n

    # Pass the variables to V_Sim.
    # Declare one set of data for this file.
    data.setNSubset(1)
    # Setup the box geomtry.
    box = v_sim.Box.new_full(box, v_sim.BoxBoundaries.PERIODIC)
    # Setting up the box geometry here make available to use the
    # converting routines to go from reduced coordinates to cartesian.
    data.setBox(box)
    # Allocate the internal arrays for these elemnts and number of nodes.
    data.allocateByNames(nElements, elements)
    for (coord, ele) in zip(coords, allElements):
      # For each node, add it with its coordinates
      data.addNodeFromElementName(ele, coord, False);
    # Set a commentry for this subset of data.
    data.setDescription(desc.strip(), 0)

    return True
  except Exception as error:
    traceback.print_exc()
    return valid
  return False

# Declare this new fileformat to V_Sim.
fmt = v_sim.DataLoader.new("VASP CNT files", ("*.cnt", "cnt.*"), False, parseCNT, 95)
v_sim.DataAtomicClass.addLoader(fmt)



#!/usr/bin/python

def dump(scene):
  scene.dump(v_sim.Dump.png_getStatic(), "out.win.png", 600, 600, None, None)
  v_sim.UiMainClass.getDefaultRendering().setCurrent(True)
  v_sim.UiMainClass.getCurrentPanel().quit(True)

def clickAt(inter, view, x, y, button = 1, shiftMod = False, controlMod = False):
  ev = v_sim.SimplifiedEvents()
  ev.x = x
  ev.y = y
  ev.button = button
  ev.buttonType = v_sim.ButtonActionId.PRESS
  inter.handleEvent(view, ev)
  ev.button = button
  ev.shiftMod = shiftMod
  ev.controlMod = controlMod
  ev.buttonType = v_sim.ButtonActionId.RELEASE
  inter.handleEvent(view, ev)

def _release(inter, view, ev):
  ev.buttonType = v_sim.ButtonActionId.RELEASE
  inter.handleEvent(view, ev)
  return FALSE
def _move(inter, view, ev, disp):
  disp[5] += 1
  x = float(disp[5]) / float(disp[4])
  fx = (2*x)**3/2 if x < 0.5 else 1-(2-2*x)**3/2
  ev.x = disp[0] + disp[2] * fx
  ev.y = disp[1] + disp[3] * fx
  ev.motion = True
  inter.handleEvent(view, ev)
  if (disp[4] == disp[5]):
    GLib.idle_add(_release, inter, view, ev)
    return False
  else:
    return True

def move(inter, view, dx, dy, button = 1, shiftMod = False, controlMod = False, duration = 500, nSteps = 50):
  ev = v_sim.SimplifiedEvents()
  ev.x = 300
  ev.y = 300
  ev.button = button
  ev.buttonType = v_sim.ButtonActionId.PRESS
  inter.handleEvent(view, ev)
  iStep = 0
  GLib.timeout_add(duration / nSteps, _move, inter, view, ev, [ev.x, ev.y, dx, dy, nSteps, iStep])

def test(scene):
  marks = scene.getMarks()
  view = scene.getGlView()
  inter = marks.get_property("interactive")
  inter.highlight(23)
  clickAt(inter, view, 342, 460, 3, shiftMod = True)
  clickAt(inter, view, 278, 377, 3)
  move(inter, view, 142, -75)
  GLib.timeout_add_seconds(1, dump, scene)

from gi.repository import GLib
  
scene = v_sim.UiMainClass.getDefaultRendering().getGlScene()
GLib.idle_add(test, scene)



#
#  libmsym.py
#  libmsym
#
#  Created by Marcus Johansson on 07/10/15.
#  Copyright (c) 2015 Marcus Johansson.
#
#  Distributed under the MIT License ( See LICENSE file or copy at http://opensource.org/licenses/MIT )
#

from ctypes import *
from ctypes.util import find_library
from enum import Enum
from copy import copy
from . import _libmsym_install_location, export

_lib = None

@export
class Error(Exception):
    def __init__(self, value, details=""):
        super().__init__(value)
        self.value=value
        self.details=details
    def __str__(self):
        return repr(self.value) + ": " + repr(self.details)
    def __repr__(self):
        return self.__str__()

try:
    import numpy as np
except ImportError:
    np = None

@export
class SymmetryOperation(Structure):

    NONE = 0,
    HORIZONTAL = 1
    VERTICAL = 2
    DIHEDRAL = 3

    IDENTITY = 0,
    PROPER_ROTATION = 1
    IMPROPER_ROTATION = 2
    REFLECTION = 3
    INVERSION = 4

    _names = ["E","C","S","\u03C3","i"]

    _proper_rotation_type_names = ["", "", "'", "''"]
    _reflection_type_names = ["", "h", "v", "d"]
    
    _fields_ = [("type", c_int),
                ("order", c_int),
                ("power", c_int),
                ("orientation", c_int),
                ("_v", (c_double*3)),
                ("conjugacy_class", c_int)]

    @property
    def vector(self):
        return self._v[0:3]

    def __str__(self):
        orientation = ""
        order = ""
        power = ""
        axis = ""
        if self.type == self.PROPER_ROTATION and self.order == 2:
            orientation = self._proper_rotation_type_names[self.orientation]
        elif self.type == self.REFLECTION:
            orientation = self._reflection_type_names[self.orientation]
            axis = " with normal vector " + repr(self.vector)

        if self.type in [self.PROPER_ROTATION, self.IMPROPER_ROTATION]:
            order = str(self.order)
            power = "^" + str(self.power)
            axis = " around " + repr(self.vector)

        return __name__ + "." + self.__class__.__name__ + "( " + self._names[self.type] + order + orientation + power + axis + ", conjugacy class: " + str(self.conjugacy_class) + " )"
    def __repr__(self):
        return self.__str__()

@export
class Element(Structure):
    _fields_ = [("_id", c_void_p),
                ("mass", c_double),
                ("_v", c_double*3),
                ("charge", c_int),
                ("_name",c_char*4)]
    
    @property
    def coordinates(self):
        return self._v[0:3]
    @coordinates.setter
    def coordinates(self, coordinates):
        self._v = (c_double*3)(*coordinates)
    @property
    def name(self):
        return self._name.decode()
    @name.setter
    def name(self, name):
        self._name = name.encode('ascii')

class _RealSphericalHarmonic(Structure):
    _fields_ = [("n", c_int),
                ("l", c_int),
                ("m", c_int)]

class _BasisFunctionUnion(Union):
    _fields_ = [("_rsh", _RealSphericalHarmonic)]

@export
class BasisFunction(Structure):
    _fields_ = [("_id", c_void_p),
                ("_type", c_int),
                ("_element", POINTER(Element)),
                ("_f", _BasisFunctionUnion),
                ("_name",c_char*8)]

    def __init__(self, element=None):
        if element == None:
            raise Error("Basis function requires an element")
        super().__init__()
        self.element = element

    def _set_element_pointer(self, element):
        self._element = pointer(element)

    @property
    def name(self):
        return self._name.decode()
    @name.setter
    def name(self, name):
        self._name = name.encode('ascii')

@export
class RealSphericalHarmonic(BasisFunction):
    def __init__(self, element=None, n=0, l=0, m=0, name=""):
        super().__init__(element=element)
        self._type = 0
        self._f._rsh.n = n
        self._f._rsh.l = l
        self._f._rsh.m = m
        self.name = name

    @property
    def n(self):
        return self._f._rsh.n
    @n.setter
    def n(self, n):
        self._f._rsh.n = n

    @property
    def l(self):
        return self._f._rsh.l
    @l.setter
    def l(self, n):
        self._f._rsh.n = l

    @property
    def m(self):
        return self._f._rsh.m
    @m.setter
    def m(self, n):
        self._f._rsh.n = m

class SALC(Structure):
    _fields_ = [("_d", c_int),
                ("_fl", c_int),
                ("_pf", POINTER(c_double)),
                ("_f", POINTER(POINTER(BasisFunction)))]

    _pf_array = None
    basis_functions = []

    def _update_basis_functions(self, basis_function_addresses, basis):
        self.basis_functions = [basis[basis_function_addresses.index(addressof(p.contents))] for p in self._f[0:self._fl]]
    
    #@property
    #def partner_functions(self):
    #    if self._pf_array is None:
    #        pf = cast(self._pf,POINTER(c_double*self._fl))
    #        self._pf_array = [f[0:self._fl] for f in pf[0:self._d]]
    #        
    #    return self._pf_array

    @property
    def partner_functions(self):
        if np is None:
            raise ImportError("numpy is not available.")
        
        if self._pf_array is None:
            self._pf_array = np.ctypeslib.as_array(self._pf, shape = (self._d,self._fl))

        return self._pf_array

        
@export
class SubrepresentationSpace(Structure):
    _fields_ = [("symmetry_species", c_int),
                ("_salc_length", c_int),
                ("_salcs", POINTER(SALC))]
    
    _salcarray = None

    @property
    def salcs(self):
        if self._salcarray is None:
            self._salcarray = self._salcs[0:self._salc_length]
        return self._salcarray

@export
class PartnerFunction(Structure):
    _fields_ = [("index", c_int),
                ("dim",c_int)]

@export
class SymmetrySpecies(Structure):
    _fields_ = [("_d", c_int),
                ("_r", c_int),
                ("_name",c_char*8)]

    @property
    def dim(self):
        return self._d

    @property
    def reducible(self):
        return self._r > 1
    
    @property
    def name(self):
        return self._name.decode()
    
class _Thresholds(Structure):
    _fields_ = [("zero", c_double),
                ("geometry", c_double),
                ("angle", c_double),
                ("equivalence", c_double),
                ("eigfact", c_double),
                ("permutation", c_double),
                ("orthogonalization", c_double)]

@export
class CharacterTable(Structure):
    _fields_ = [("_d", c_int),
                ("_classc", POINTER(c_int)),
                ("_sops", POINTER(POINTER(SymmetryOperation))),
                ("_s", POINTER(SymmetrySpecies)),
                ("_table", POINTER(c_double))]

    _table_array = None
    _class_count_array = None
    _symmetry_species = None

    @property
    def table(self):
        if np is None:
            raise ImportError("numpy is not available.")
        
        if self._table_array is None:
            self._table_array = np.ctypeslib.as_array(self._table, shape = (self._d,self._d))

        return self._table_array

    @property
    def class_count(self):
        if self._class_count_array is None:
            self._class_count_array = self._classc[0:self._d]
        return self._class_count_array

    def _update_symmetry_operations(self, symmetry_operations):
        addresses = [addressof(sop) for sop in symmetry_operations]
        self.symmetry_operations = [symmetry_operations[addresses.index(addressof(sop.contents))] for sop in self._sops[0:self._d]]

    @property
    def symmetry_species(self):
        if self._symmetry_species is None:
            self._symmetry_species = self._s[0:self._d]
            
        return self._symmetry_species
        
class _ReturnCode(c_int):

    SUCCESS = 0
    INVALID_INPUT = -1
    INVALID_CONTEXT = -2
    INVALID_THRESHOLD = -3
    INVALID_ELEMENTS = -4
    INVALID_BASIS = -5
    INVALID_POINT_GROUP = -6
    INVALID_EQUIVALENCE_SET = -7
    INVALID_PERMUTATION = -8
    INVALID_GEOMETRY = -9
    INVALID_CHARACTER_TABLE = -10
    INVALID_SUBSPACE = -11
    INVALID_SUBGROUPS = -12
    INVALID_AXES = -13
    SYMMETRY_ERROR = -14
    PERMUTATION_ERROR = -15
    POINT_GROUP_ERROR = -16
    SYMMETRIZATION_ERROR = -17
    SUBSPACE_ERROR = -18

    def __str__(self):    
        #init is not called on the return type so we can't contruct data on creation, don't decode details here, may be too late
        error_string = _lib.msymErrorString(self.value).decode()
        return repr(error_string)

    def __repr__(self):
        return self.__str__()


def init(library_location=None):

    if(library_location is None):
        raise Error("Cannot find libmsym shared library")

    global _lib
    
    _lib = CDLL(library_location)
    
    _Context = POINTER(type('msym_context', (Structure,), {}))
     
    _lib.msymErrorString.argtypes = [c_int]
    _lib.msymErrorString.restype = c_char_p
    
    _lib.msymCreateContext.restype = _Context
    _lib.msymCreateContext.argtypes = []

    _lib.msymGetDefaultThresholds.restype = POINTER(_Thresholds)
    _lib.msymGetDefaultThresholds.argtypes = []

    _lib.msymSetThresholds.restype = _ReturnCode
    _lib.msymSetThresholds.argtypes = [_Context, POINTER(_Thresholds)]

    _lib.msymReleaseContext.restype = _ReturnCode
    _lib.msymReleaseContext.argtypes = [_Context]

    _lib.msymGetErrorDetails.restype = c_char_p
    _lib.msymGetErrorDetails.argtypes = []
    
    _lib.msymFindSymmetry.restype = _ReturnCode
    _lib.msymFindSymmetry.argtypes = [_Context]

    _lib.msymSetPointGroupByName.restype = _ReturnCode
    _lib.msymSetPointGroupByName.argtypes = [_Context, c_char_p]

    _lib.msymGetPointGroupName.restype = _ReturnCode
    _lib.msymGetPointGroupName.argtypes = [_Context, c_int, c_char_p]

    _lib.msymSetElements.restype = _ReturnCode
    _lib.msymSetElements.argtypes = [_Context, c_int, POINTER(Element)]

    _lib.msymGenerateElements.restype = _ReturnCode
    _lib.msymGenerateElements.argtypes = [_Context, c_int, POINTER(Element)]

    _lib.msymGetElements.restype = _ReturnCode
    _lib.msymGetElements.argtypes = [_Context, POINTER(c_int), POINTER(POINTER(Element))]

    _lib.msymGetSymmetryOperations.restype = _ReturnCode
    _lib.msymGetSymmetryOperations.argtypes = [_Context, POINTER(c_int), POINTER(POINTER(SymmetryOperation))]

    _lib.msymSymmetrizeElements.restype = _ReturnCode
    _lib.msymSymmetrizeElements.argtypes = [_Context]

    _lib.msymSetBasisFunctions.restype = _ReturnCode
    _lib.msymSetBasisFunctions.argtypes = [_Context, c_int, POINTER(BasisFunction)]

    _lib.msymGetBasisFunctions.restype = _ReturnCode
    _lib.msymGetBasisFunctions.argtypes = [_Context, POINTER(c_int), POINTER(POINTER(BasisFunction))]

    _lib.msymGetSubrepresentationSpaces.restype = _ReturnCode
    _lib.msymGetSubrepresentationSpaces.argtypes = [_Context, POINTER(c_int), POINTER(POINTER(SubrepresentationSpace))]

    _lib.msymGetCharacterTable.restype = _ReturnCode
    _lib.msymGetCharacterTable.argtypes = [_Context, POINTER(POINTER(CharacterTable))]

    if np is None:
        _SALCsMatrix = c_void_p
        _SALCsSpecies = POINTER(c_int)
        _NPDArray = POINTER(c_double)
    else:
        _SALCsMatrix = np.ctypeslib.ndpointer(dtype=np.float64, ndim=2, flags='C_CONTIGUOUS')
        _SALCsSpecies = np.ctypeslib.ndpointer(dtype=np.int32, ndim=1, flags='C_CONTIGUOUS')
        _NPDArray = np.ctypeslib.ndpointer(dtype=np.float64, ndim=1, flags='C_CONTIGUOUS')

    _lib.msymSymmetrySpeciesComponents.restype = _ReturnCode
    _lib.msymSymmetrySpeciesComponents.argtypes = [_Context, c_int, _NPDArray, c_int, _NPDArray]

    _lib.msymGetSALCs.restype = _ReturnCode
    _lib.msymGetSALCs.argtypes = [_Context, c_int, _SALCsMatrix, _SALCsSpecies, POINTER(PartnerFunction)]

    _lib.msymSymmetrizeWavefunctions.restype = _ReturnCode
    _lib.msymSymmetrizeWavefunctions.argtypes = [_Context, c_int, _SALCsMatrix, _SALCsSpecies, POINTER(PartnerFunction)]


_libmsym_location = find_library('msym')

if _libmsym_location is None:
    _libmsym_location = _libmsym_install_location

if not (_libmsym_location is None):
    init(_libmsym_location)

@export
class Context(object):

    _ctx = None
    
    def __init__(self, elements=[], basis_functions=[], point_group=""):
        if(_lib is None):
            raise Error("Shared library not loaded")
            
        self._elements = []
        self._basis_functions = []
        self._point_group = None
        self._subrepresentation_spaces = None
        self._character_table = None
        self._ctx = _lib.msymCreateContext()
        if not self._ctx:
            raise RuntimeError('Failed to create libmsym context')
        pthresholds = _lib.msymGetDefaultThresholds()
        default_thresholds = pthresholds.contents        
        if default_thresholds is None:
            raise RuntimeError('Failed get libmsym default thresholds')
        self._thresholds = copy(default_thresholds)
        if len(elements) > 0:
            self._set_elements(elements)
        if len(basis_functions) > 0:
            self._set_basis_functions(basis_functions)
        if len(point_group) > 0:
            self._set_point_group(point_group)
            self.find_symmetry()
            
    def __del__(self):
        
        self._destruct()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self._destruct()

    def _destruct(self): 
        if self._ctx:
            _lib.msymReleaseContext(self._ctx)
        self._ctx = None

    @staticmethod
    def _assert_success(error):
        if not error.value == _ReturnCode.SUCCESS:
            raise Error(error, details = _lib.msymGetErrorDetails().decode())

    def _get_basis_function_addresses(self):
        if not self._ctx:
            raise RuntimeError
        cbfs = POINTER(BasisFunction)()
        csize = c_int(0)
        self._assert_success(_lib.msymGetBasisFunctions(self._ctx,byref(csize),byref(cbfs)))
        return [addressof(bf) for bf in cbfs[0:csize.value]]

    def _set_elements(self, elements):
        if not self._ctx:
            raise RuntimeError
        self._subrepresentation_spaces = None
        self._character_table = None
        self._salcs = None
        self._basis_functions = []
        size = len(elements)
        element_array = (Element*size)(*elements)
        self._assert_success(_lib.msymSetElements(self._ctx, size, element_array))
        self._element_array = element_array
        self._elements = elements

    def _set_point_group(self, point_group):
        if not self._ctx:
            raise RuntimeError
        self._subrepresentation_spaces = None
        self._character_table = None
        self._salcs = None
        cname = c_char_p(point_group.encode('ascii'))
        self._assert_success(_lib.msymSetPointGroupByName(self._ctx, cname))
        self._update_symmetry_operations()

    def _set_basis_functions(self, basis_functions):
        if not self._ctx:
            raise RuntimeError
        self._subrepresentation_spaces = None
        self._salcs = None
        size = len(basis_functions)
        for bf in basis_functions:
            bf._set_element_pointer(self._element_array[self._elements.index(bf.element)])
            
        self._assert_success(_lib.msymSetBasisFunctions(self._ctx, size, (BasisFunction*size)(*basis_functions)))
        self._basis_functions = basis_functions
        if not self._point_group is None:
            self._update_symmetry_operations()
            self._update_character_table()
        
    def _update_elements(self):
        if not self._ctx:
            raise RuntimeError
        celements = POINTER(Element)()
        csize = c_int(0)
        self._assert_success(_lib.msymGetElements(self._ctx,byref(csize),byref(celements)))
        self._elements_array = celements
        self._elements = celements[0:csize.value]

    def _update_symmetry_operations(self):
        if not self._ctx:
            raise RuntimeError
        csops = POINTER(SymmetryOperation)()
        csize = c_int(0)
        self._assert_success(_lib.msymGetSymmetryOperations(self._ctx,byref(csize),byref(csops)))
        self._symmetry_operations = csops[0:csize.value]

    def _update_point_group(self):
        if not self._ctx:
            raise RuntimeError
        cname = (c_char*8)()
        self._assert_success(_lib.msymGetPointGroupName(self._ctx,sizeof(cname),cname))
        self._point_group = cname.value.decode()
        
    def _update_subrepresentation_spaces(self):
        if not self._ctx:
            raise RuntimeError
        basis_function_addresses = self._get_basis_function_addresses()
        csrs = POINTER(SubrepresentationSpace)()
        csize = c_int(0)
        self._assert_success(_lib.msymGetSubrepresentationSpaces(self._ctx,byref(csize),byref(csrs)))
        srs = csrs[0:csize.value]
        for s in srs:
            for salc in s.salcs:
                salc._update_basis_functions(basis_function_addresses, self._basis_functions)
        self._subrepresentation_spaces = srs

    def _update_character_table(self):
        if not self._ctx:
            raise RuntimeError
        cct = POINTER(CharacterTable)()
        self._assert_success(_lib.msymGetCharacterTable(self._ctx,byref(cct)))
        self._character_table = cct.contents
        self._character_table._update_symmetry_operations(self._symmetry_operations)

    def _update_salcs(self):
        if not self._ctx:
            raise RuntimeError
        if np is None:
            raise ImportError("numpy is not available.")
        csize = len(self._basis_functions)
        partner_functions = (PartnerFunction*csize)()
        salcs = np.zeros((csize,csize),dtype=np.float64)
        species = np.zeros((csize),dtype=np.int32)
        self._assert_success(_lib.msymGetSALCs(self._ctx,csize,salcs,species,partner_functions))
        self._salcs = (salcs, species, partner_functions[0:csize])

    def set_thresholds(self, **kwargs):
        for key in kwargs.keys():
            if not key in ['zero','geometry','angle','equivalence',
                           'eigfact','permutation','orthogonalization']:
                raise Error('Unrecognized threshold argument')
            setattr(self._thresholds, key, kwargs[key])
        self._assert_success(_lib.msymSetThresholds(self._ctx, pointer(self._thresholds)))
        
        
    @property
    def elements(self):
        return self._elements

    @elements.setter
    def elements(self, elements):
        self._set_elements(elements)

    @property
    def basis_functions(self):
        return self._basis_functions

    @basis_functions.setter
    def basis_functions(self, basis_functions):
        self._set_basis_functions(basis_functions)

    @property
    def point_group(self):
        return self._point_group

    @point_group.setter
    def point_group(self, point_group):
        self._set_point_group(point_group)

    @property
    def symmetry_operations(self):
        return self._symmetry_operations

    def find_symmetry(self):
        if not self._ctx:
            raise RuntimeError
        self._assert_success(_lib.msymFindSymmetry(self._ctx))
        self._update_point_group()
        self._update_symmetry_operations()
        return self._point_group

    def symmetrize_elements(self):
        if not self._ctx:
            raise RuntimeError
        cerror = c_double(0)
        self._assert_success(_lib.msymSymmetrizeElements(self._ctx, byref(cerror)))
        self._update_elements()
        return self._elements

    @property
    def subrepresentation_spaces(self):
        if self._subrepresentation_spaces is None:
            self._update_subrepresentation_spaces()
         
        return self._subrepresentation_spaces

    @property
    def character_table(self):
        if self._character_table is None:
            self._update_character_table()
         
        return self._character_table

    @property
    def salcs(self):
        if self._salcs is None:
            self._update_salcs()

        return self._salcs

    def symmetrize_wavefunctions(self,m):
        if not self._ctx:
            raise RuntimeError
        if np is None:
            raise ImportError("numpy is not available.")
        csize = len(self._basis_functions)
        (d1,d2) = m.shape
        if not (d1 == csize and d2 == csize):
            raise ValueError("Must provide a " + str(csize) + "x" + str(csize))
        wf = np.ascontiguousarray(m, dtype=np.float64)
        partner_functions = (PartnerFunction*csize)()
        species = np.zeros((csize),dtype=np.int32)
        self._assert_success(_lib.msymSymmetrizeWavefunctions(self._ctx,csize,wf,species,partner_functions))
        return (wf, species, partner_functions[0:csize])

    def generate_elements(self, elements):
        if not self._ctx:
            raise RuntimeError
        self._subrepresentation_spaces = None
        self._character_table = None
        self._salcs = None
        self._element_array = None
        self._basis_functions = []
        self._elements = []
        size = len(elements)
        element_array = (Element*size)(*elements)
        self._assert_success(_lib.msymGenerateElements(self._ctx, size, element_array))
        self._update_elements()
        return self._elements

    def symmetry_species_components(self, wf):
       
        wf_size = len(wf)
        if not wf_size == len(self.basis_functions):
            raise ValueError("Must provide an array of length " + str(len(self.basis_functions)))
        species_size = self.character_table._d
        species = np.zeros((species_size),dtype=np.float64)
        wf = np.ascontiguousarray(wf, dtype=np.float64)
        self._assert_success(_lib.msymSymmetrySpeciesComponents(self._ctx, wf_size, wf, species_size, species))
        return species
        








#!/usr/bin/env python

import sys, os, tempfile
import unittest

import gi
gi.require_version('v_sim', '3.8')
from gi.repository import GLib, v_sim

import signals

class TestLoader(unittest.TestCase):
  def setUp(self):
    super(TestLoader, self).setUp()
    self.addTypeEqualityFunc(float, self.fuzzyFloat)

  def fuzzyFloat(self, a, b, msg = None):
    if abs(b-a) > 1e-8:
      raise self.failureException(msg if msg is not None else "%g != %g (d = %g)" % (a, b, abs(a-b)))
        
  def _loader(self, l, path, expectation = {}):
    data = v_sim.DataAtomic.new(path, l)
    try:
      ok = data.load(0, None)
    except:
      ok = False
    self.assertEqual(ok, expectation["success"])

    if ok:
      for (k, v) in expectation.items():
        if k.startswith("data."):
          #print eval(k), v
          self.assertEqual(eval(k), v)

  def _tempFile(self, string):
    f = tempfile.NamedTemporaryFile()
    f.write(string)
    f.flush()
    return f

  def test_ascii_file(self):
    self._loader(v_sim.DataLoader.ascii_getStatic(),
                 os.path.join(os.path.dirname(sys.argv[0]),
                                "../../examples/demo.ascii"),
                   {"success": True,
                    "data.getNNodes()": 172,
                    "data.getNElements(True)": 2,
                    "data.containsElement(v_sim.Element.retrieveFromName(\"H\")[0])": False,
                    "data.containsElement(v_sim.Element.lookup(\"Ni\"))": True,
                    "data.containsElement(v_sim.Element.lookup(\"Au\"))": True,
                    "data.getBox().getUnit()": v_sim.Units.ANGSTROEM,
                    "data.getBox().getBoundary()": v_sim.BoxBoundaries.PERIODIC,
                    "data.getBox().getPeriodicity()": [True, True, True],
                   })
  def test_ascii_wrong_file(self):
    self._loader(v_sim.DataLoader.ascii_getStatic(),
                 os.path.join(os.path.dirname(sys.argv[0]),
                                "../../examples/demo.xyz"),
                   {"success": False})
  def test_ascii_unit_bohr(self):
    with self._tempFile("""test
10 0 10
0  0 10
#keyword: bohr
0 0 0 Si
""") as f:
      self._loader(v_sim.DataLoader.ascii_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getUnit()": v_sim.Units.BOHR})
  def test_ascii_unit_angstroem(self):
    with self._tempFile("""test
10 0 10
0  0 10
#keyword: Angstroem
0 0 0 Si
""") as f:
      self._loader(v_sim.DataLoader.ascii_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getUnit()": v_sim.Units.ANGSTROEM})
  def test_ascii_unit_undefined(self):
    with self._tempFile("""test
10 0 10
0  0 10
0 0 0 Si
""") as f:
      self._loader(v_sim.DataLoader.ascii_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getUnit()": v_sim.Units.UNDEFINED})
  def test_ascii_box_periodic(self):
    with self._tempFile("""test
10 0 10
0  0 10
#keyword: periodic
0 0 0 Si
""") as f:
      self._loader(v_sim.DataLoader.ascii_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getBoundary()": v_sim.BoxBoundaries.PERIODIC})
    with self._tempFile("""test
10 0 10
0  0 10
0 0 0 Si
""") as f:
      self._loader(v_sim.DataLoader.ascii_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getBoundary()": v_sim.BoxBoundaries.PERIODIC})
  def test_ascii_box_free(self):
    with self._tempFile("""test
10 0 10
0  0 10
#keyword: freeBC
0 0 0 Si
""") as f:
      self._loader(v_sim.DataLoader.ascii_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getBoundary()": v_sim.BoxBoundaries.FREE})
  def test_ascii_box_surface(self):
    with self._tempFile("""test
10 0 10
0  0 10
#keyword: surface
0 0 0 Si
""") as f:
      self._loader(v_sim.DataLoader.ascii_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getBoundary()": v_sim.BoxBoundaries.SURFACE_ZX})
  def test_ascii_box(self):
    with self._tempFile("""test
1 2 3
4 5 6
0 0 0 Si
""") as f:
      self._loader(v_sim.DataLoader.ascii_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getGeometry(v_sim.BoxVector.DXX)": 1.,
                            "data.getBox().getGeometry(v_sim.BoxVector.DYX)": 2.,
                            "data.getBox().getGeometry(v_sim.BoxVector.DYY)": 3.,
                            "data.getBox().getGeometry(v_sim.BoxVector.DZX)": 4.,
                            "data.getBox().getGeometry(v_sim.BoxVector.DZY)": 5.,
                            "data.getBox().getGeometry(v_sim.BoxVector.DZZ)": 6.})
  def test_ascii_box_angdeg(self):
    with self._tempFile("""test
1 2 3
90 90 90
#keyword: angdeg
0 0 0 Si
""") as f:
      self._loader(v_sim.DataLoader.ascii_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getGeometry(v_sim.BoxVector.DXX)": 1.,
                            "data.getBox().getGeometry(v_sim.BoxVector.DYX)": 0.,
                            "data.getBox().getGeometry(v_sim.BoxVector.DYY)": 2.,
                            "data.getBox().getGeometry(v_sim.BoxVector.DZX)": 0.,
                            "data.getBox().getGeometry(v_sim.BoxVector.DZY)": 0.,
                            "data.getBox().getGeometry(v_sim.BoxVector.DZZ)": 3.})
  def test_ascii_reduced(self):
    with self._tempFile("""test
1 0 2
0 0 3
#keyword: reduced
0.5 0.75 0.333333333333 Si
""") as f:
      self._loader(v_sim.DataLoader.ascii_getStatic(),
                   f.name, {"success": True,
                            "data.getNodeCoordinates(data.getFromId(0), False)": (0.5, 1.5, 1.)})
  def test_ascii_coord(self):
    with self._tempFile("""test
1 0 2
0 0 3
1 2 3 Si
""") as f:
      self._loader(v_sim.DataLoader.ascii_getStatic(),
                   f.name, {"success": True,
                            "data.getNodeCoordinates(data.getFromId(0), False)": (1., 2., 3.)})
  def test_ascii_coord_user(self):
    with self._tempFile("""test
1 0 2
0 0 3
#keyword: freeBC
1 2 3 Si
""") as f:
      self._loader(v_sim.DataLoader.ascii_getStatic(),
                   f.name, {"success": True,
                            "data.getNodeCoordinates(data.getFromId(0), True)": (1., 2., 3.),
                            "data.getNodeCoordinates(data.getFromId(0), False)": (0., 0., 0.)})
  def test_ascii_props(self):
    with self._tempFile("""test
1 0 2
0 0 3
1 2 3 Si hello
0 0 0 C {"IGSpin": -1}
""") as f:
      self._loader(v_sim.DataLoader.ascii_getStatic(),
                   f.name, {"success": True,
                            "data.getNodeLabelAt(data.getFromId(0))": "hello",
                            "data.getNodeProperties(\"IGSpin\").getAt(data.getFromId(1))[1]": -1})
  def test_ascii_energy(self):
    with self._tempFile("""test
1 0 2
0 0 3
#metaData: totalEnergy = 10.
1 2 3 Si
#
""") as f:
      self._loader(v_sim.DataLoader.ascii_getStatic(),
                   f.name, {"success": True,
                            "data.get_property(\"totalEnergy\")": 10.})
  def test_ascii_energy_ht(self):
    with self._tempFile("""test
1 0 2
0 0 3
#metaData: totalEnergy = 1. Ht
1 2 3 Si
#
""") as f:
      self._loader(v_sim.DataLoader.ascii_getStatic(),
                   f.name, {"success": True,
                            "data.get_property(\"totalEnergy\")": 27.21138386})
  def test_ascii_forces(self):
    with self._tempFile("""test
1 0 2
0 0 3
1 2 3 Si
0 0 0 Si
#metaData: forces = [\
# 1, 2, 3, \
# 4, 5, 6]
#
""") as f:
      self._loader(v_sim.DataLoader.ascii_getStatic(),
                   f.name, {"success": True,
                            "data.getForces(False).getAt(data.getFromId(0))": [1, 2, 3],
                            "data.getForces(False).getAt(data.getFromId(1))": [4, 5, 6]})
  def test_ascii_forces_none(self):
    with self._tempFile("""test
1 0 2
0 0 3
1 2 3 Si
0 0 0 Si
""") as f:
      self._loader(v_sim.DataLoader.ascii_getStatic(),
                   f.name, {"success": True,
                            "data.getForces(False)": None,
                            "data.getForces(True).getAt(data.getFromId(0))": [0, 0, 0]})
    
  def test_xyz_file(self):
    self._loader(v_sim.DataLoader.xyz_getStatic(),
                 os.path.join(os.path.dirname(sys.argv[0]),
                                "../../examples/demo.xyz"),
                   {"success": True,
                    "data.getNNodes()": 950,
                    "data.getNElements(True)": 2,
                    "data.containsElement(v_sim.Element.retrieveFromName(\"H\")[0])": False,
                    "data.containsElement(v_sim.Element.lookup(\"Ni\"))": True,
                    "data.containsElement(v_sim.Element.lookup(\"C\"))": True,
                    "data.getBox().getUnit()": v_sim.Units.UNDEFINED,
                    "data.getBox().getBoundary()": v_sim.BoxBoundaries.FREE,
                    "data.getBox().getPeriodicity()": [False, False, False],
                   })
  def test_xyz_wrong_file(self):
    self._loader(v_sim.DataLoader.xyz_getStatic(),
                 os.path.join(os.path.dirname(sys.argv[0]),
                                "../../examples/demo.ascii"),
                   {"success": False})
  def test_xyz_unit_bohr(self):
    with self._tempFile("""1 bohr

Si 0 0 0
""") as f:
      self._loader(v_sim.DataLoader.xyz_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getUnit()": v_sim.Units.BOHR})
  def test_xyz_unit_angstroem(self):
    with self._tempFile("""1 angstroem

Si 0 0 0
""") as f:
      self._loader(v_sim.DataLoader.xyz_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getUnit()": v_sim.Units.ANGSTROEM})
  def test_xyz_unit_undefined(self):
    with self._tempFile("""1

Si 0 0 0
""") as f:
      self._loader(v_sim.DataLoader.xyz_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getUnit()": v_sim.Units.UNDEFINED})
  def test_xyz_box_periodic(self):
    with self._tempFile("""1
periodic 10 10 10 # coming from a surface file
Si 0 0 0
""") as f:
      self._loader(v_sim.DataLoader.xyz_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getBoundary()": v_sim.BoxBoundaries.PERIODIC})
  def test_xyz_box_free(self):
    with self._tempFile("""1

Si 0 0 0
""") as f:
      self._loader(v_sim.DataLoader.xyz_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getBoundary()": v_sim.BoxBoundaries.FREE})
  def test_xyz_box_surface(self):
    with self._tempFile("""1
surface 10 10 10 # coming from a periodic file
Si 0 0 0
""") as f:
      self._loader(v_sim.DataLoader.xyz_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getBoundary()": v_sim.BoxBoundaries.SURFACE_ZX})
  def test_xyz_box(self):
    with self._tempFile("""1
periodic 1 2 3
Si 0 0 0
""") as f:
      self._loader(v_sim.DataLoader.xyz_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getGeometry(v_sim.BoxVector.DXX)": 1.,
                            "data.getBox().getGeometry(v_sim.BoxVector.DYX)": 0.,
                            "data.getBox().getGeometry(v_sim.BoxVector.DYY)": 2.,
                            "data.getBox().getGeometry(v_sim.BoxVector.DZX)": 0.,
                            "data.getBox().getGeometry(v_sim.BoxVector.DZY)": 0.,
                            "data.getBox().getGeometry(v_sim.BoxVector.DZZ)": 3.})
  def test_xyz_coord(self):
    with self._tempFile("""1
periodic 10 10 10
Si 1 2 3
""") as f:
      self._loader(v_sim.DataLoader.xyz_getStatic(),
                   f.name, {"success": True,
                            "data.getNodeCoordinates(data.getFromId(0), False)": (1., 2., 3.)})
  def test_xyz_coord_user(self):
    with self._tempFile("""1

Si 1 2 3
""") as f:
      self._loader(v_sim.DataLoader.xyz_getStatic(),
                   f.name, {"success": True,
                            "data.getNodeCoordinates(data.getFromId(0), True)": (1., 2., 3.),
                            "data.getNodeCoordinates(data.getFromId(0), False)": (0., 0., 0.)})
  def test_xyz_props(self):
    with self._tempFile("""2

Si 1 2 3 hello
C 0 0 0 {"IGSpin": -1}
""") as f:
      self._loader(v_sim.DataLoader.xyz_getStatic(),
                   f.name, {"success": True,
                            "data.getNodeLabelAt(data.getFromId(0))": "hello",
                            "data.getNodeProperties(\"IGSpin\").getAt(data.getFromId(1))[1]": -1})
  def test_xyz_forces(self):
    with self._tempFile("""2

Si 1 2 3
Si 0 0 0
FoRceS
Si 1 2 3
Si 4 5 6
""") as f:
      self._loader(v_sim.DataLoader.xyz_getStatic(),
                   f.name, {"success": True,
                            "data.getForces(False).getAt(data.getFromId(0))": [1, 2, 3],
                            "data.getForces(False).getAt(data.getFromId(1))": [4, 5, 6]})
  def test_xyz_forces_none(self):
    with self._tempFile("""2

Si 1 2 3
Si 0 0 0
""") as f:
      self._loader(v_sim.DataLoader.xyz_getStatic(),
                   f.name, {"success": True,
                            "data.getForces(False)": None,
                            "data.getForces(True).getAt(data.getFromId(0))": [0, 0, 0]})
    
  def test_yaml_file(self):
    self._loader(v_sim.DataLoader.yaml_getStatic(),
                 os.path.join(os.path.dirname(sys.argv[0]),
                                "../../examples/cinchonidine.yaml"),
                   {"success": True,
                    "data.getNNodes()": 44,
                    "data.getNElements(True)": 4,
                    "data.containsElement(v_sim.Element.retrieveFromName(\"Ni\")[0])": False,
                    "data.containsElement(v_sim.Element.lookup(\"H\"))": True,
                    "data.containsElement(v_sim.Element.lookup(\"C\"))": True,
                    "data.containsElement(v_sim.Element.lookup(\"O\"))": True,
                    "data.containsElement(v_sim.Element.lookup(\"N\"))": True,
                    "data.getBox().getUnit()": v_sim.Units.BOHR,
                    "data.getBox().getBoundary()": v_sim.BoxBoundaries.FREE,
                    "data.getBox().getPeriodicity()": [False, False, False],
                   })
  def test_yaml_wrong_file(self):
    self._loader(v_sim.DataLoader.yaml_getStatic(),
                 os.path.join(os.path.dirname(sys.argv[0]),
                                "../../examples/demo.ascii"),
                   {"success": False})
  def test_yaml_unit_bohr(self):
    with self._tempFile("""positions:
  - H: [0., 0., 0.]
""") as f:
      self._loader(v_sim.DataLoader.yaml_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getUnit()": v_sim.Units.BOHR})
    with self._tempFile("""units: bohr
positions:
  - H: [0., 0., 0.]
""") as f:
      self._loader(v_sim.DataLoader.yaml_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getUnit()": v_sim.Units.BOHR})
  def test_yaml_unit_angstroem(self):
    with self._tempFile("""units: angstroem
positions:
  - H: [0., 0., 0.]
""") as f:
      self._loader(v_sim.DataLoader.yaml_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getUnit()": v_sim.Units.ANGSTROEM})
  def test_yaml_box_periodic(self):
    with self._tempFile("""cell: [10., 10., 10.]
positions:
  - H: [0., 0., 0.]
""") as f:
      self._loader(v_sim.DataLoader.yaml_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getBoundary()": v_sim.BoxBoundaries.PERIODIC})
  def test_yaml_box_free(self):
    with self._tempFile("""positions:
  - H: [0., 0., 0.]
""") as f:
      self._loader(v_sim.DataLoader.yaml_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getBoundary()": v_sim.BoxBoundaries.FREE})
    with self._tempFile("""cell: [.inf, .inf, .inf]
positions:
  - H: [0., 0., 0.]
""") as f:
      self._loader(v_sim.DataLoader.yaml_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getBoundary()": v_sim.BoxBoundaries.FREE})
  def test_yaml_box_surface(self):
    with self._tempFile("""cell: [10., .inf, 10.]
positions:
  - H: [0., 0., 0.]
""") as f:
      self._loader(v_sim.DataLoader.yaml_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getBoundary()": v_sim.BoxBoundaries.SURFACE_ZX})
  def test_yaml_box(self):
    with self._tempFile("""cell: [1., 2., 3.]
positions:
  - H: [0., 0., 0.]
""") as f:
      self._loader(v_sim.DataLoader.yaml_getStatic(),
                   f.name, {"success": True,
                            "data.getBox().getGeometry(v_sim.BoxVector.DXX)": 1.,
                            "data.getBox().getGeometry(v_sim.BoxVector.DYX)": 0.,
                            "data.getBox().getGeometry(v_sim.BoxVector.DYY)": 2.,
                            "data.getBox().getGeometry(v_sim.BoxVector.DZX)": 0.,
                            "data.getBox().getGeometry(v_sim.BoxVector.DZY)": 0.,
                            "data.getBox().getGeometry(v_sim.BoxVector.DZZ)": 3.})
  def test_yaml_coord(self):
    with self._tempFile("""cell: [10., 10., 10.]
positions:
  - Si: [1., 2., 3.]
""") as f:
      self._loader(v_sim.DataLoader.yaml_getStatic(),
                   f.name, {"success": True,
                            "data.getNodeCoordinates(data.getFromId(0), False)": (1., 2., 3.)})
  def test_yaml_coord_user(self):
    with self._tempFile("""positions:
  - Si: [1., 2., 3.]
""") as f:
      self._loader(v_sim.DataLoader.yaml_getStatic(),
                   f.name, {"success": True,
                            "data.getNodeCoordinates(data.getFromId(0), True)": (1., 2., 3.),
                            "data.getNodeCoordinates(data.getFromId(0), False)": (0., 0., 0.)})
  def test_yaml_props(self):
    with self._tempFile("""positions:
  - {Si: [1., 2., 3.]}
  - {C: [0., 0., 0.], IGSpin: -1}
""") as f:
      self._loader(v_sim.DataLoader.yaml_getStatic(),
                   f.name, {"success": True,
                            "data.getNodeLabelAt(data.getFromId(0))": None,
                            "data.getNodeProperties(\"IGSpin\").getAt(data.getFromId(1))[1]": -1})
  def test_yaml_forces(self):
    with self._tempFile("""positions:
  - Si: [1., 2., 3.]
  - C: [0., 0., 0.]
forces:
  Values:
  - Si: [1., 2., 3.]
  - C: [4., 5., 6.]
""") as f:
      self._loader(v_sim.DataLoader.yaml_getStatic(),
                   f.name, {"success": True,
                            "data.getForces(False).getAt(data.getFromId(0))": [1, 2, 3],
                            "data.getForces(False).getAt(data.getFromId(1))": [4, 5, 6]})
  def test_yaml_forces_none(self):
    with self._tempFile("""positions:
  - Si: [1., 2., 3.]
  - C: [0., 0., 0.]
""") as f:
      self._loader(v_sim.DataLoader.yaml_getStatic(),
                   f.name, {"success": True,
                            "data.getForces(False)": None,
                            "data.getForces(True).getAt(data.getFromId(0))": [0, 0, 0]})

  def test_d3_file(self):
    self._loader(v_sim.DataLoader.d3_getStatic(),
                 os.path.join(os.path.dirname(sys.argv[0]),
                                "../../examples/aluminium.d3"),
                   {"success": True,
                    "data.getNNodes()": 134,
                    "data.getNElements(True)": 1,
                    "data.containsElement(v_sim.Element.retrieveFromName(\"Ni\")[0])": False,
                    "data.containsElement(v_sim.Element.lookup(\"Al\"))": True,
                    "data.getBox().getUnit()": v_sim.Units.UNDEFINED,
                    "data.getBox().getBoundary()": v_sim.BoxBoundaries.PERIODIC,
                    "data.getBox().getGeometry(v_sim.BoxVector.DXX)": 24.72848892,
                    "data.getBox().getGeometry(v_sim.BoxVector.DYX)": -12.36424446,
                    "data.getBox().getGeometry(v_sim.BoxVector.DYY)": 21.41550064,
                    "data.getBox().getGeometry(v_sim.BoxVector.DZX)": 0.,
                    "data.getBox().getGeometry(v_sim.BoxVector.DZY)": 0.,
                    "data.getBox().getGeometry(v_sim.BoxVector.DZZ)": 24.
                   })

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python

import libmsym as msym, numpy as np, argparse, random, sys



def read_xyz(fin):
    length = int(fin.readline())
    comment = fin.readline()[:-1]
    elements = []
    for i in range(0,length):
        line = fin.readline().split()
        elements.append(msym.Element(name = line[0], coordinates = map(float, line[1:4])))

    return (elements, comment)

def write_xyz(fout, elements, comment):
    fout.write("%d\n%s\n" % (len(elements), comment))
    for e in elements:
        v = e.coordinates
        fout.write("%s %.14f %.14f %.14f\n" % (e.name, *v))

parser = argparse.ArgumentParser()
parser.add_argument('infile', type=argparse.FileType('r'))
parser.add_argument('outfile', type=argparse.FileType('w'))
args = parser.parse_args()

(elements, comment) = read_xyz(args.infile)
 
def set_basis(element):
    basis_function = msym.RealSphericalHarmonic(element = element, name = "1s")
    element.basis_functions = [basis_function]
    return basis_function

basis_functions = [set_basis(e) for e in elements]


#msym.init(library_location='/path_to_libmsym_library/libmsym.dylib') # e.g. for OS X without libmsym in dload path

#with msym.Context() as ctx:
#    ctx.elements = elements
#    point_group = ctx.find_symmetry()
#    selements = ctx.symmetrize_elements()
#    write_xyz(args.outfile, selements, comment + " symmetrized by libmsym according to point group " + point_group)
 
with msym.Context(elements = elements, basis_functions = basis_functions) as ctx:
    point_group = ctx.find_symmetry()
    selements = ctx.symmetrize_elements()
    write_xyz(args.outfile, selements, comment + " symmetrized by libmsym according to point group " + point_group)
    ctx.symmetry_operations #symmetry operations
    ctx.subrepresentation_spaces # subspace
    ctx.subrepresentation_spaces[0].symmetry_species #symmetry species of space (index into character table)
    ctx.subrepresentation_spaces[0].salcs # salcs that span space
    ctx.subrepresentation_spaces[0].salcs[0].basis_functions # basis functions for salc    
    ctx.subrepresentation_spaces[0].salcs[0].partner_functions # numpy array of partner functions expressed in terms of basis_functions coefficients

    ctx.character_table.table # table as numpy array
    ctx.character_table.symmetry_operations # representative symmetry operations
    ctx.character_table.symmetry_species # symmetry species
    ctx.character_table.symmetry_species[0].dim  # dimensionality of symmetry species
    ctx.character_table.symmetry_species[0].name # name of symmetry species e.g. A2g

    somefunc = np.zeros((len(basis_functions)),dtype=np.float64)
    for i in range(0,len(somefunc)):
        somefunc[i] = i

    species_components = ctx.symmetry_species_components(somefunc)
    
    species = ctx.character_table.symmetry_species

    print(somefunc)
    for i, c in enumerate(species_components):
        print(str(c) + species[i].name)
    
    #matrix version of the above, as well as wave function symmetrization
    (matrix, species, partners) = ctx.salcs
    (d,d) = matrix.shape
    print(matrix)
    print(species)
    print([(p.index, p.dim) for p in partners])
    indexes = [x for x in range(0,d)]
    random.shuffle(indexes)
    matrix = matrix[indexes,:]
    matrix += 0.01
    print(matrix)
    (_same_matrix, species,partners) = ctx.symmetrize_wavefunctions(matrix)
    print(matrix)
    print(species)
    print([(p.index, p.dim) for p in partners])

    #generating elements
    ctx.point_group = "D6h"
    gen_elements = [msym.Element(name = "C", coordinates = [1.443524, 0.0,0.0]), msym.Element(name = "H", coordinates = [2.568381, 0.0, 0.0])]
    benzene = ctx.generate_elements(gen_elements)
    maxcomp = max([max(e.coordinates) for e in benzene])
    print(len(benzene),"\nbenzene")
    for e in benzene:
        vec = np.asarray(e.coordinates)
        vec[vec < maxcomp*sys.float_info.epsilon] = 0
        print(e.name, vec[0],vec[1],vec[2])
    

#with msym.Context(elements = elements, point_group = "T") as ctx:
#    point_group = ctx.point_group
#    selements = ctx.symmetrize_elements()
#    write_xyz(args.outfile, selements, comment + " symmetrized by libmsym according to point group " + point_group)







#!/usr/bin/env python
# -*- coding: utf-8 -*-
from gi.repository import v_sim
import cProfile

render = v_sim.UiMainClass.getDefaultRendering()
data = render.getVisuData()

# Example with integrated iterators (still an issue with the
# stop criterion)
rad = {}
for it in data.__iterElements__():
  rad[it.element.getName()] = v_sim.RenderingAtomic.getRadius(it.element)
#print "Radii: ", rad

def hit((x1,y1,z1), r1, (x2,y2,z2), r2):
  return ( (x1-x2)**2 + (y1-y2)**2 + (z1-z2)**2 < (r1 + r2)**2 )

# With Python iterators.
collision = set()

def byPairs(data, collision):
  for (it1, it2) in data.__iterByPairs__():
    if hit(data.getNodeCoordinates(it1.node), rad[it1.element.getName()], \
           data.getNodeCoordinates(it2.node), rad[it2.element.getName()]):
      collision.add(int(it1.node.number))
      collision.add(int(it2.node.number))

def doubleLoop(data, collision):
  for it1 in data:
    for it2 in data:
      if it1.node.number > it2.node.number and \
         hit(data.getNodeCoordinates(it1.node), rad[it1.element.getName()], \
             data.getNodeCoordinates(it2.node), rad[it2.element.getName()]):
        collision.add(int(it1.node.number))
        collision.add(int(it2.node.number))

def byCopy(data, collision):
  nodes = data.getAllNodePositions()
  for (c1, i1, e1) in nodes:
    for (c2, i2, e2) in nodes:
      if i1 > i2 and hit(c1, rad[e1], c2, rad[e2]):
        collision.add(int(i1))
        collision.add(int(i2))

cProfile.run('doubleLoop(data, collision)')
cProfile.run('byPairs(data, collision)')
cProfile.run('byCopy(data, collision)')

marks = render.getMarks()
if marks.setHighlightedList(tuple(collision), v_sim.MarksStatus.SET):
  v_sim.Object.redraw("My script")

#v_sim.UiMainClass.getCurrentPanel().quit(True)



#!/usr/bin/env python

from gi.repository import GLib

class Listener():
    def __init__(self, obj, signal):
        self.trigger = 0
        self.target = obj
        self.signal = obj.connect(signal, self.onTrigger)
    def __enter__(self):
        return self
    def __exit__(self, type, value, traceback):
        self.target.handler_disconnect(self.signal)
        self.target = None

    def onTrigger(self, obj, args):
        self.trigger += 1

    def triggered(self):
        ctx = GLib.MainContext.default()
        while (ctx.pending()):
            ctx.iteration(False)
        return self.trigger

    def reset(self):
        self.trigger = 0



#!/usr/bin/env python
from gi.repository import v_sim

# Load the file to test.
data = v_sim.Data.new()
data.addFile("collisions.ascii",0,None)
v_sim.visuBasicLoad_dataFromFile(data,None,0)

# Parse the configuration files.
v_sim.visuBasicParse_configFiles()
# Or read only the given file.
#v_sim.visuConfigFileLoad(v_sim.CONFIGFILE_RESOURCE, "v_sim.res", data)


# Example of loop to create a dictionnary associating element name and radius.
rad = {}
dataIter = data.iterNew()
data.iterStart(dataIter)
while (dataIter.element != None):
  rad[dataIter.element.name] = v_sim.renderingAtomicGet_radius(dataIter.element)
  data.iterNextElement(dataIter)

# Example with integrated iterators (still an issue with the
# stop criterion)
try:
  for it in data.__iterElements__():
    rad[it.element.name] = v_sim.renderingAtomicGet_radius(it.element)
except RuntimeError:
  pass
print "Radii: ", rad


def hit(c1, r1, c2, r2):
  return ( (c1.x-c2.x)**2 + (c1.y-c2.y)**2 + (c1.z-c2.z)**2 < (r1 + r2) ** 2 )

# With Python iterators.
##collision = []
##try:
##  for it1 in data:
##    try:
##      for it2 in data:
##        if it1.node.number > it2.node.number and \
##	     hit(data.getNodeCoordinates(it1.node), rad[it1.element.name], \
##		 data.getNodeCoordinates(it2.node), rad[it2.element.name]):
##	  collision.append((int(it1.node.number), int(it2.node.number)))
##    except RuntimeError:
##      pass
##except RuntimeError:
##  pass
##print collision

# With V_Sim iterators
print "Collision detection..."
collision = []
it1 = data.iterNew()
it2 = data.iterNew()
data.iterStart(it1)
while (it1.node is not None):
  data.iterStart(it2)
  while (it2.node is not None):
    if it1.node.number > it2.node.number and \
	 hit(data.getNodeCoordinates(it1.node), rad[it1.element.name], \
	     data.getNodeCoordinates(it2.node), rad[it2.element.name]):
      collision.append((int(it1.node.number), int(it2.node.number)))
    data.iterNext(it2)
  data.iterNext(it1)
print collision



#!/usr/bin/env python
from gi.repository import v_sim

# Load the file to test.
data=v_sim.Data.new()
data.addFile("diff.ascii",0,None)
v_sim.visuBasicLoad_dataFromFile(data,None,0)

# Parse the configuration files.
v_sim.visuBasicParse_configFiles()

def prox(data, c1, c2, rad):
  r1 = data.convertXYZToReduced(c1)
  r2 = data.convertXYZToReduced(c2)
  for dx in range(-1, 2):
    for dy in range(-1, 2):
      for dz in range(-1, 2):
        r2_trans = v_sim.Coord.__new__(v_sim.Coord)
	r2_trans.x = r2.x + dx
	r2_trans.y = r2.y + dy
	r2_trans.z = r2.z + dz
	c2_trans = data.convertReducedToXYZ(r2_trans)
	dist2 = (c1.x-c2_trans.x)**2 + (c1.y-c2_trans.y)**2 + (c1.z-c2_trans.z)**2
	if dist2 <= (rad) ** 2:
	  return True
  return False

##RAD = input('Choose a radius: ')
##NODE = input('Choose a node id: ')
RAD = 5.
NODE = 216
print "All nodes further than ",RAD," from node ",NODE," will be hidden."

coord0 = data.getNodeCoordinates(data.getNodeFromNumber(NODE))

dataIter = data.iterNew()
data.iterStart(dataIter)
while (dataIter.node is not None):
	if prox(data, coord0, data.getNodeCoordinates(dataIter.node), RAD):
		print "Node ",dataIter.node.number," is visible"
		dataIter.node.setVisibility(True)
	else:
		print "Node ",dataIter.node.number," is hidden"
		dataIter.node.setVisibility(False)
	data.iterNext(dataIter)

#----------------------DUMP----------------------#
print '\n\t---DUMP dans du resultat dans diffOut.ascii---\n'

outFilename = "diffOut.png"
##outFilename = "diffOut.ascii"

dumps = v_sim.visuDumpGet_allModules()
for dump in dumps:
	if dump.fileType.match(outFilename):
		if dump.bitmap:
			ctx = v_sim.PixmapContext.new(450, 450)
			v_sim.openGLInit_context();

			view = data.getOpenGLView()
			view.camera.length0 = data.getBoxLengths()[0];

			v_sim.openGLModelize(view.camera);
			data.setSizeOfView(450, 450)
			v_sim.openGLProject(view.window, view.camera,
					    data.getBoxLengths()[1])

			v_sim.visuExtensions_rebuildAllLists(data)
			v_sim.openGL_reDraw(None, data)

			image = v_sim.visuOpenGLGet_pixmapData(450, 450, True)
		else:
			image = None

		# The exportation routine.
		dump.callWriteFunc(outFilename, 450, 450, data, image, None, None)
		
		if dump.bitmap:
			ctx.free()
		break



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: set fileencoding=utf-8

## EXTRAITS DE LA LICENCE
## Copyright CEA, contributeurs : Damien CALISTE, laboratoire L_Sim, (2012-2012)
## Adresse mèl :
## CALISTE, damien P caliste AT cea P fr.

## Ce logiciel est un programme informatique servant à visualiser des
## structures atomiques dans un rendu pseudo-3D. 
## Ce logiciel est régi par la licence CeCILL soumise au droit français et
## respectant les principes de diffusion des logiciels libres. Vous pouvez
## utiliser, modifier et/ou redistribuer ce programme sous les conditions
## de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA 
## sur le site "http://www.cecill.info".
## Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
## pris connaissance de la licence CeCILL, et que vous en avez accepté les
## termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).

## LICENCE SUM UP
## Copyright CEA, contributors: Damien CALISTE, L_Sim laboratory, (2012-2012)
## E-mail address:
## CALISTE, damien P caliste AT cea P fr.

## This software is a computer program whose purpose is to visualize atomic
## configurations in 3D.
## This software is governed by the CeCILL  license under French law and
## abiding by the rules of distribution of free software.  You can  use, 
## modify and/ or redistribute the software under the terms of the CeCILL
## license as circulated by CEA, CNRS and INRIA at the following URL
## "http://www.cecill.info". 
## The fact that you are presently reading this means that you have had
## knowledge of the CeCILL license and that you accept its terms. You can
## find a copy of this licence shipped with this software at
## Documentation/licence.en.txt.

from gi.repository import v_sim
from gi.repository import Gtk, GLib

def onMeasure(item, marks, nodeId):
  if marks.setInfos(nodeId, True):
    v_sim.Object.redrawForce("rightMenu script")

def onClean(item, marks, nodeId):
  if marks.setInfos(nodeId, False):
    v_sim.Object.redrawForce("rightMenu script")

def buildMenu(inter, node):
  menu = Gtk.Menu.new()

  marks = v_sim.UiMainClass.getDefaultRendering().getMarks()

  # The default contents (single node case).
  if node is not None:
    item = Gtk.MenuItem.new_with_label("Measure node neighbourhood")
    item.connect("activate", onMeasure, marks, node.number)
    menu.append(item)
    item = Gtk.MenuItem.new_with_label("Remove measures from node")
    item.connect("activate", onClean, marks, node.number)
    item.set_sensitive(marks.getActive(node.number))
    menu.append(item)

  # The user added items.
  for (lblNone, lblSingle, lblSelection, callback, kwargs) in inter.actionList.values():
    if node is None:
      item = Gtk.MenuItem.new_with_label(lblNone)
      item.connect("activate", callback, None, kwargs)
    else:
      item = Gtk.MenuItem.new_with_label(lblSingle)
      item.connect("activate", callback, node.number, kwargs)
    menu.append(item)
  
  return menu

def menuPosition(menu, (x, y)):
  return (x, y, True)

def onActionMenu(inter, x, y, node):
  print node
  menu = buildMenu(inter, node)
  menu.show_all()
  menu.popup(None, None, menuPosition, (x, y), 0, Gtk.get_current_event_time())

# Define a right clic menu on rendering window.
inter = v_sim.UiRenderingWindowClass.getInteractive()
if 'actionMenuId' in globals():
  inter.disconnect(actionMenuId)
actionMenuId = inter.connect("menu", onActionMenu)

# Add new methods to the interactive object of the rendering window.
def addNodeAction(self, id, labelNone, labelSingle, labelSelection, callback,
                  kwargs = {}):
  self.actionList[id] = (labelNone, labelSingle, labelSelection, callback, kwargs)

def removeNodeAction(self, id):
  self.actionList.pop(id, None)

# Add a list of custom action to inter.
if not(hasattr(v_sim.Interactive, "actionList")):
  setattr(v_sim.Interactive, "actionList", {})
  # Add methods to review this list.
  v_sim.Interactive.addNodeAction = addNodeAction
  v_sim.Interactive.removeNodeAction = removeNodeAction



#!/usr/bin/env python

import unittest

import gi
gi.require_version('v_sim', '3.8')
from gi.repository import GLib, v_sim

import signals

class TestBox(unittest.TestCase):
    def setUp(self):
        super(TestBox, self).setUp()
        self.addTypeEqualityFunc(float, self.fuzzyFloat)

    def fuzzyFloat(self, a, b, msg = None):
        if abs(b-a) > 6e-8:
            raise self.failureException(msg)

    def _boundary(self, bc):
        # Creation
        box = v_sim.Box.new((1., 2., 3., 4., 5., 6.), bc)
        self.assertEqual(box.getBoundary(), bc)
        box = v_sim.Box.new_full((1., 2., 3., 4., 5., 6., 7., 8., 9.), bc)
        self.assertEqual(box.getBoundary(), bc)
        # Setter
        box = v_sim.Box.new((1., 2., 3., 4., 5., 6.), v_sim.BoxBoundaries.FREE)
        with signals.Listener(box, "notify::boundary") as boundary:
            self.assertEqual(boundary.triggered(), 0)
            self.assertEqual(box.setBoundary(bc), bc != v_sim.BoxBoundaries.FREE)
            self.assertEqual(box.getBoundary(), bc)
            self.assertEqual(boundary.triggered(), 1 if bc != v_sim.BoxBoundaries.FREE else 0)
            box.set_property("boundary", v_sim.BoxBoundaries.FREE)
            self.assertEqual(box.getBoundary(), v_sim.BoxBoundaries.FREE)
            self.assertEqual(boundary.triggered(), 2 if bc != v_sim.BoxBoundaries.FREE else 1)

    def test_boundaries(self):
        self._boundary(v_sim.BoxBoundaries.FREE)
        self._boundary(v_sim.BoxBoundaries.WIRE_X)
        self._boundary(v_sim.BoxBoundaries.WIRE_Y)
        self._boundary(v_sim.BoxBoundaries.WIRE_Z)
        self._boundary(v_sim.BoxBoundaries.SURFACE_XY)
        self._boundary(v_sim.BoxBoundaries.SURFACE_YZ)
        self._boundary(v_sim.BoxBoundaries.SURFACE_ZX)
        self._boundary(v_sim.BoxBoundaries.PERIODIC)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: set fileencoding=utf-8

## EXTRAITS DE LA LICENCE
## Copyright CEA, contributeurs : Damien CALISTE, laboratoire L_Sim, (2012-2012)
## Adresse mèl :
## CALISTE, damien P caliste AT cea P fr.

## Ce logiciel est un programme informatique servant à visualiser des
## structures atomiques dans un rendu pseudo-3D. 
## Ce logiciel est régi par la licence CeCILL soumise au droit français et
## respectant les principes de diffusion des logiciels libres. Vous pouvez
## utiliser, modifier et/ou redistribuer ce programme sous les conditions
## de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA 
## sur le site "http://www.cecill.info".
## Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
## pris connaissance de la licence CeCILL, et que vous en avez accepté les
## termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).

## LICENCE SUM UP
## Copyright CEA, contributors: Damien CALISTE, L_Sim laboratory, (2012-2012)
## E-mail address:
## CALISTE, damien P caliste AT cea P fr.

## This software is a computer program whose purpose is to visualize atomic
## configurations in 3D.
## This software is governed by the CeCILL  license under French law and
## abiding by the rules of distribution of free software.  You can  use, 
## modify and/ or redistribute the software under the terms of the CeCILL
## license as circulated by CEA, CNRS and INRIA at the following URL
## "http://www.cecill.info". 
## The fact that you are presently reading this means that you have had
## knowledge of the CeCILL license and that you accept its terms. You can
## find a copy of this licence shipped with this software at
## Documentation/licence.en.txt.

from gi.repository import v_sim
from gi.repository import Gtk

import numpy

from matplotlib.figure import Figure
from matplotlib.backends.backend_cairo import FigureCanvasCairo as FigureCanvas
from matplotlib.backends.backend_cairo import RendererCairo as Renderer

dpi = 100
fig_width = 3
fig_height = 2

def mplCurveDraw(widget, cr, f, renderer):
  renderer.gc.ctx = cr
  w = widget.get_allocated_width()
  h = widget.get_allocated_height()
  cr.translate(max(0, (w - fig_width * dpi) * 0.5),
               -max(0, (h - fig_height * dpi) * 0.5))
  renderer.set_width_height(w, h)
  f.draw(renderer)

def onTogglePick(widget, interPick, panel, wdx, wdy, wdz, wdx_ref, wdy_ref, wdz_ref):
  render = v_sim.UiMainClass.getDefaultRendering()
  if (widget.get_active()):
    handle = interPick.connect("node-selection", onNodeSelected, panel, widget,
                               wdx, wdy, wdz, wdx_ref, wdy_ref, wdz_ref)
    interPick.set_data("pick", handle)
    render.pushInteractive(interPick)
    render.pushMessage("Pick a node with the mouse")
  else:
    interPick.disconnect(interPick.get_data("pick"))
    render.popInteractive(interPick)
    render.popMessage()

def onNodeSelected(inter, kind, node0, node1, node2, panel, toggle,
                   entryX, entryY, entryZ, entryX_ref, entryY_ref, entryZ_ref):
  if not(kind == v_sim.InteractivePick.SELECTED):
    return

  data = panel.getData()
  (x, y, z) = data.getNodeCoordinates(node0)
  if entryX_ref is not None:
    entryX.setValue(x - entryX_ref.getValue())
  else:
    entryX.setValue(x)
  if entryY_ref is not None:
    entryY.setValue(y - entryY_ref.getValue())
  else:
    entryY.setValue(y)
  if entryZ_ref is not None:
    entryZ.setValue(z - entryZ_ref.getValue())
  else:
    entryZ.setValue(z)
  toggle.set_active(False)

def onDraw(widget, area, mplFig, panel, combo, (ox, oy, oz), (dx, dy, dz)):
  # We get the scalar field.
  it = combo.get_active_iter()
  if it is None:
    return
  model = combo.get_model()
  (field,) = model.get(it, v_sim.UiSurfacesFieldId.POINTER)
  data = panel.getData()

  # We setup the data to iterate over.
  orig = numpy.array(data.convertXYZToReduced((ox.getValue(), oy.getValue(), oz.getValue())))
  vect = numpy.array(data.convertXYZToReduced((dx.getValue(), dy.getValue(), dz.getValue())))
  norm = 1. / numpy.sqrt(vect[0] * vect[0] + vect[1] * vect[1] + vect[2] * vect[2])
  vect *= norm
  l0 = max(-orig[0] / max(vect[0], 1e-6), -orig[1] / max(vect[1], 1e-6), -orig[2] / max(vect[2], 1e-6))
  l1 = min((1. - orig[0]) / max(vect[0], 1e-6), (1. - orig[1]) / max(vect[1], 1e-6), (1. - orig[2]) / max(vect[2], 1e-6))
  orig0 = numpy.array(data.convertReducedToXYZ(tuple(orig + l0 * vect)))
  vect0 = numpy.array(data.convertReducedToXYZ(tuple(orig + l1 * vect))) - orig0
  norm = numpy.sqrt(vect0[0] * vect0[0] + vect0[1] * vect0[1] + vect0[2] * vect0[2])
  
  # We setup the data to be plotted.
  a = mplFig.add_subplot(111)
  x = numpy.arange(0.0, 1.0, 0.01)
  y = numpy.arange(0.0, 1.0, 0.01)
  for i in range(100):
    point = orig0 + x[i] * vect0
    (valid, val) = field.getValue(tuple(point), (0,0,0))
    if valid:
      y[i] = val
  x *= norm
  a.plot(x,y)
  # We ask for redraw of the widget.
  area.queue_draw()

def panelCreateSelection(panel, vbox, interPick, label,
                         wdx_ref = None, wdy_ref = None, wdz_ref = None):
  # The line to choose the origin/orientation.
  hbox = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 5)
  vbox.pack_start(hbox, False, False, 0)

  lbl = Gtk.Label.new(label)
  hbox.pack_start(lbl, False, False, 0)

  wdx = v_sim.UiNumericalEntry.new(0.)
  wdx.set_width_chars(6)
  hbox.pack_start(wdx, True, True, 0)
  wdy = v_sim.UiNumericalEntry.new(0.)
  wdy.set_width_chars(6)
  hbox.pack_start(wdy, True, True, 0)
  wdz = v_sim.UiNumericalEntry.new(0.)
  wdz.set_width_chars(6)
  hbox.pack_start(wdz, True, True, 0)

  wd = Gtk.ToggleButton.new()
  wd.add(Gtk.Image.new_from_stock(Gtk.STOCK_FIND, Gtk.IconSize.MENU))
  wd.connect("toggled", onTogglePick, interPick, panel,
             wdx, wdy, wdz, wdx_ref, wdy_ref, wdz_ref)
  hbox.pack_start(wd, False, False, 0)

  return (wdx, wdy, wdz)

def panelCreateWidgets():
  panel = v_sim.UiPanel.new("mytools", "Customized tools", "My tools")
  panel.setDockable(True)
  panel.attach(v_sim.UiPanelClass.getCommandPanel());

  vbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
  panel.add(vbox)

  lbl = Gtk.Label.new("<b>Interpolate data along lines</b>")
  lbl.set_alignment(0., 0.5)
  lbl.set_use_markup(True)
  vbox.pack_start(lbl, False, False, 0)

  # The line to choose a data field.
  hbox = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 5)
  vbox.pack_start(hbox, False, False, 0)

  lbl = Gtk.Label.new("Choose a data field:")
  hbox.pack_start(lbl, False, False, 0)

  combo = Gtk.ComboBox.new()
  combo.set_model(v_sim.UiPanel.surfaces_getFields())
  hbox.pack_start(combo, True, True, 0)
  renderer = Gtk.CellRendererText.new()
  combo.pack_start(renderer, False)
  renderer.set_property("xalign", 1.0)
  combo.add_attribute(renderer, "markup", v_sim.UiSurfacesFieldId.LABEL)

  interPick = v_sim.Interactive.new(v_sim.InteractiveId.PICK)

  # The line to choose the origin.
  (origx, origy, origz) = panelCreateSelection(panel, vbox, interPick,
                                               "Choose the origin:")

  # The line to choose the direction.
  (dirx, diry, dirz) = panelCreateSelection(panel, vbox, interPick,
                                            "Choose the direction:",
                                            wdx_ref = origx, wdy_ref = origy,
                                            wdz_ref = origz)
  dirx.setValue(1.)
  diry.setValue(1.)
  dirz.setValue(1.)

  # The drawing area for the curve.
  hbox = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 0)
  vbox.pack_start(hbox, True, True, 0)

  scrolled = Gtk.ScrolledWindow.new(None, None)
  scrolled.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
  hbox.pack_start(scrolled, True, True, 0)

  f = Figure(figsize=(fig_width,fig_height), dpi = dpi)
  canvas = FigureCanvas(f)
  f.set_canvas(canvas)
  renderer = Renderer(dpi = dpi)

  area = Gtk.DrawingArea.new()
  area.set_size_request(fig_width * dpi, fig_height * dpi)
  area.connect("draw", mplCurveDraw, f, renderer)
  scrolled.add_with_viewport(area)

  # The toolbar for action on the curve.
  wd = Gtk.Toolbar.new()
  wd.set_orientation(Gtk.Orientation.VERTICAL)
  wd.set_style(Gtk.ToolbarStyle.ICONS)
  wd.set_icon_size(Gtk.IconSize.SMALL_TOOLBAR)
  hbox.pack_start(wd, False, False, 0)

  item = Gtk.ToolButton.new_from_stock(Gtk.STOCK_REFRESH)
  item.connect("clicked", onDraw, area, f, panel, combo,
               (origx, origy, origz), (dirx, diry, dirz))
  wd.insert(item, -1)

  return (f, panel, interPick)

# We create the widgets and plot the stuff.
(mplFig, panel, i) = panelCreateWidgets()
panel.show_all()




#!/usr/bin/env python

from gi.repository import v_sim
import math, time

render = v_sim.uiMainClass_getDefaultRendering()
data = render.getVisuData()

camera = data.getOpenGLView().camera
(theta0, gross0) = (camera.theta, camera.gross)

dumps = v_sim.dump_getAllModules()
for dump in dumps:
  if dump.fileType.match("a.png"):
    format = dump

NB = 100.
for i in range(0, int(NB) + 1):
  data.setAngleOfView(theta0 + 360. * i / NB, 0, 0, v_sim.CAMERA_THETA)
  data.setZoomOfView(gross0 + 0.5 * math.sin(i * math.pi * 2. / NB))
  v_sim.object_redraw("My script")
  #time.sleep(0.01)
  #render.dump(format, "rotation%03d.png" % i, 450, 450, None, None)




#!/usr/bin/python

(valid, nLines) = v_sim.ConfigFile.saveResourcesToXML("./tmp.res.xml", None)

if not(valid):
  raise ValueError("Resources not written")

print nLines

v_sim.UiMainClass.getCurrentPanel().quit(True)



#!/usr/bin/python

def dump(scene):
  scene.dump(v_sim.Dump.png_getStatic(), "out.win.png", 600, 600, None, None)
  v_sim.UiMainClass.getDefaultRendering().setCurrent(True)
  v_sim.UiMainClass.getCurrentPanel().quit(True)

def clickAt(inter, view, x, y, button = 1, shiftMod = False, controlMod = False):
  ev = v_sim.SimplifiedEvents()
  ev.x = x
  ev.y = y
  ev.button = button
  ev.buttonType = v_sim.ButtonActionId.PRESS
  inter.handleEvent(view, ev)
  ev.button = button
  ev.shiftMod = shiftMod
  ev.controlMod = controlMod
  ev.buttonType = v_sim.ButtonActionId.RELEASE
  inter.handleEvent(view, ev)

def test(scene):
  marks = scene.getMarks()
  view = scene.getGlView()
  inter = marks.get_property("interactive")
  inter.highlight(23)
  clickAt(inter, view, 420, 429, 3, shiftMod = True)
  clickAt(inter, view, 351, 392, 3)
  scene.getData().reload(None)
  GLib.idle_add(dump, scene)

from gi.repository import GLib
  
scene = v_sim.UiMainClass.getDefaultRendering().getGlScene()
GLib.idle_add(test, scene)



#/usr/bin/env python

scene = v_sim.UiMainClass.getDefaultRendering().getGlScene()
scene.dump(v_sim.Dump.png_getStatic(), "out.win.png", 600, 600, None, None)
v_sim.UiMainClass.getCurrentPanel().quit(True)



#!/usr/bin/env python

import sys
import numpy
from scipy import misc

rad = sys.argv[1]

ref = misc.imread(sys.argv[2]).astype(numpy.float32)[:,:,0:3]

for i in range(3, len(sys.argv)):
  img = misc.imread(sys.argv[i]).astype(numpy.float32)[:,:,0:3]
  ssd = numpy.sqrt((ref - img) ** 2)
  print "%15g %15g" % (ssd.ptp(), ssd.sum() / ssd.shape[0] / ssd.shape[1]),
  
  ssd *= 10.
  ssd.clip(0.,255.)
  misc.imsave(rad + ".diff%d.png" % (i - 3), ssd.astype(numpy.uint8))

print "# %s peak-to-peak average" % rad



#!/usr/bin/env python

import sys, os
import unittest

import gi
gi.require_version('v_sim', '3.8')
from gi.repository import GLib, v_sim

import signals

class TestToolFiles(unittest.TestCase):
  def ascii_text(self, path = None, string = None):
    f = v_sim.Files.new()
    if path is not None:
      try:
        f.open(path)
      except GLib.Error as e:
        if e.code != GLib.FileError.FAILED:
          raise
        return
    if string is not None:
      f.fromMemory(string)
    (s, l, pos) = f.read_line_string()
    self.assertEqual(s, GLib.IOStatus.NORMAL)
    self.assertEqual(l.str, "Some ASCII data.\n")
    (s, l, pos) = f.read_line_string()
    self.assertEqual(s, GLib.IOStatus.NORMAL)
    self.assertEqual(l.str, "\n")
    (s, l, pos) = f.read_line_string()
    self.assertEqual(s, GLib.IOStatus.EOF)
    self.assertEqual(l.str, '')
    s = f.rewind()
    self.assertEqual(s, GLib.IOStatus.NORMAL)
    (s, l, pos) = f.read_line_string()
    self.assertEqual(s, GLib.IOStatus.NORMAL)
    self.assertEqual(l.str, "Some ASCII data.\n")

  def test_ascii_text(self):
    self.ascii_text(None, """Some ASCII data.

""")
    self.ascii_text(os.path.join(os.path.dirname(sys.argv[0]), "ascii_text"))
    self.ascii_text(os.path.join(os.path.dirname(sys.argv[0]), "ascii_text.bz2"))

  def fortran_data(self, path):
    f = v_sim.Files.new()
    try:
      f.fortran_open(path)
    except GLib.Error as e:
      if e.code != GLib.FileError.FAILED:
        raise
      return
    (ok, endian) = f.fortran_testEndianness(128)
    self.assertTrue(ok)
    self.assertEqual(endian, v_sim.FortranEndianId.KEEP)
    (ok, lbl) = f.fortran_readString(128, endian, False)
    self.assertTrue(ok)
    self.assertEqual(lbl, "Some title text.")
    ok = f.fortran_checkFlag(128, endian)
    self.assertTrue(ok)
    (ok, n) = f.fortran_readFlag(endian)
    self.assertTrue(ok)
    self.assertEqual(n, 12)
    (ok, nat) = f.fortran_readInteger(endian)
    self.assertTrue(ok)
    self.assertEqual(nat, 3)
    (ok, dbl) = f.fortran_readDouble(endian)
    self.assertTrue(ok)
    self.assertEqual(dbl, 1.)
    ok = f.fortran_checkFlag(n, endian)
    self.assertTrue(ok)
    (ok, rxyz) = f.fortran_readDoubleArray(9, endian, True)
    self.assertTrue(ok)
    self.assertEqual(rxyz[0:3], [1., 2., 3.])
    self.assertEqual(rxyz[3:6], [7., 8., 9.])

  def test_fortran_data(self):
    self.fortran_data(os.path.join(os.path.dirname(sys.argv[0]), "fortran_data"))
    self.fortran_data(os.path.join(os.path.dirname(sys.argv[0]), "fortran_data.bz2"))

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

import math, random
import gobject
import gtk, v_sim

def delete_event(widget, event, data=None):
    return False

def destroy(widget, data=None):
    gtk.main_quit()

def reset(widget, v_sim, handles):
  stop(widget, handles)
  v_sim.set(None)

def stop(widget, handles):
  if (handles[0] is not None):
    gobject.source_remove(handles[0])
  if (handles[1] is not None):
    gobject.source_remove(handles[1])
  handles[0] = None
  handles[1] = None

def load(widget, v_sim, data, nodes, C, angle, handles):
  if (handles[0] is not None):
    gobject.source_remove(handles[0])
  if (handles[1] is not None):
    gobject.source_remove(handles[1])

  v_sim.set(data)

  handles[0] = gobject.timeout_add(10, turn, angle, v_sim, data, nodes)
  handles[1] = gobject.timeout_add(1000, add, data, angle, C, nodes)
  

def turn(angle, v_sim, data, nodes):
  angle[1] = math.pi * 2 / 3 + 0.5 * math.sin(angle[0] * 2)
  nodes[0].set(coord=(1.2 * math.sin(angle[1]) * (math.cos(angle[0])) + 2.5, \
                      1.2 * math.sin(angle[1]) * (math.sin(angle[0])) + 2.5, \
                      1.2 * math.cos(angle[1]) + 2))
  nodes[1].set(coord=(1.2 * math.sin(angle[1]) * (math.cos(angle[0] + math.pi)) + 2.5, \
                      1.2 * math.sin(angle[1]) * (math.sin(angle[0] + math.pi)) + 2.5, \
                      1.2 * math.cos(angle[1]) + 2), emit = True)
  if (nodes[2].isValid()):
    nodes[2].set(coord=(2.5, 2.5, 4 + math.sin(angle[1] + math.pi/4)))
  data.createAllNodes()
  v_sim.redraw()
  angle[0] += math.pi / 100
  if (angle[0] > 2 * math.pi):
    angle[0] = 0
  print data.getPopulation()
  return True

def add(data, angle, C, nodes):
  if (nodes[2].isValid()):
    data.delNodes(nodes[2])
  else:
    nodes[2] = data.addNode(C, (2.5, 2.5, 4. + math.sin(angle[1] + math.pi/4)), emit = True)
  return True

def start(window, handles):
    render = v_sim.main(width=400)
    window.get_data("hbox").pack_start(render, False, False, 0)

    angle = [0., math.pi * 2 / 3]
    O = v_sim.element("O", rgba=(1., 0.2, 0., 1.))
    #O.set(rendered = False)
    H = v_sim.element("H", rgba=(0.2, 0.4, 0.5, 0.3))
    C = v_sim.element("C", rgba=(0.1, 0.1, 0.1, 1.))

    data = v_sim.data()
    data.setPopulation({O:1, H:2, C:1})
    data.addNode(O, (2.5, 2.5, 2))
    nodes = []
    nodes.append(data.addNode(H, (1.5, 2.5, 1)))
    nodes.append(data.addNode(H, (3.5, 2.5, 1)))
    nodes.append(v_sim.node(data, 3))
    data.setBox((5., 0., 5., 0., 0., 5.), True)

    plane = v_sim.plane(normal = (1,0,0), distance = 3, rgba = (1,0.5,0.5,1))
    data.setPlanes((plane,))

    window.get_data("play").connect("clicked", load, v_sim, data, nodes, C, angle, handles)
    window.get_data("stop").connect("clicked", stop, handles)
    window.get_data("reset").connect("clicked", reset, v_sim, handles)

    return False

bouncing = """
O = v_sim.element("O", rgba=(1., 0.2, 0., 1.))
#O.set(rendered = False)
H = v_sim.element("H", rgba=(0.2, 0.4, 0.5, 0.3))
C = v_sim.element("C", rgba=(0.1, 0.1, 0.1, 1.))

data = v_sim.data()
data.setPopulation({O:1, H:2, C:1})
data.addNode(O, (2.5, 2.5, 2))
data.addNode(H, (1.5, 2.5, 1))
data.addNode(H, (3.5, 2.5, 1))
data.setBox((5., 0., 5., 0., 0., 5.), True)
"""

handles = [None, None]

# Set the GTK interface.
window = gtk.Window(gtk.WINDOW_TOPLEVEL)
window.connect("delete_event", delete_event)
window.connect("destroy", destroy)
hbox = gtk.HBox()
window.add(hbox)
vbox = gtk.VBox()
vbox.set_border_width(7)
hbox.pack_start(vbox, True, True, 0)

wd = gtk.Label("<span size=\"large\"><b>Demonstrate Python bindings for V_Sim:</b></span>")
wd.set_use_markup(True)
wd.set_padding(10, 0)
vbox.pack_start(wd, False, False, 20)

scroll = gtk.ScrolledWindow()
scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
scroll.set_shadow_type(gtk.SHADOW_ETCHED_OUT)
vbox.pack_start(scroll, True, True, 0)
buf = gtk.TextBuffer()
buf.set_text(bouncing)
wd = gtk.TextView(buf)
wd.set_size_request(300, -1)
wd.set_editable(False)
scroll.add(wd)

com = gtk.HBox()
vbox.pack_end(com, False, False, 5)
wd = gtk.Button(stock = gtk.STOCK_MEDIA_PLAY)
window.set_data("play", wd)
com.pack_end(wd, False, False, 0)
wd = gtk.Button(stock = gtk.STOCK_MEDIA_STOP)
window.set_data("stop", wd)
com.pack_end(wd, False, False, 0)
wd = gtk.Button(stock = gtk.STOCK_REMOVE)
com.pack_start(wd, False, False, 0)
window.set_data("reset", wd)

window.set_data("hbox", hbox)

window.show_all()

gobject.idle_add(start, window, handles)

gtk.main()




#!/usr/bin/env python
from gi.repository import v_sim
import cProfile

render = v_sim.uiMainClass_getDefaultRendering()
data = render.getVisuData()

def prox(data, (x0, y0, z0), c, rad):
  (u, v, w) = data.convertXYZToReduced(c)
  for dx in range(-1, 2):
    for dy in range(-1, 2):
      for dz in range(-1, 2):
        (x, y, z) = data.convertReducedToXYZ((u + dx, v + dy, w + dz))
	dist2 = (x - x0)**2 + (y - y0)**2 + (z - z0)**2
	if dist2 <= (rad) ** 2:
	  return True
  return False

def isolate(RAD, NODE):
  coord0 = data.getNodeCoordinates(data.getNodeFromNumber(NODE))
  dataIter = data.iterNew()
  data.iterStart(dataIter)
  while (dataIter.node is not None):
    if prox(data, coord0, data.getNodeCoordinates(dataIter.node), RAD):
      print "Node ",dataIter.node.number," is visible"
      dataIter.node.setVisibility(True)
    else:
      print "Node ",dataIter.node.number," is hidden"
      dataIter.node.setVisibility(False)
    data.iterNext(dataIter)
  data.createAllNodes()


##RAD = input('Choose a radius: ')
##NODE = input('Choose a node id: ')
RAD = 5.
NODE = 21
print "All nodes further than ",RAD," from node ",NODE," will be hidden."
cProfile.run('isolate(RAD, NODE)')
v_sim.object_redraw(None)

#----------------------DUMP----------------------#
outFilename = "diffOut.png"
print '\n\t---DUMP dans du resultat dans %s---\n' % outFilename

##outFilename = "diffOut.ascii"

dumps = v_sim.dump_getAllModules()
for dump in dumps:
  if dump.fileType.match(outFilename):
    render.dump(dump, outFilename, 450, 450, None, None)



#
#  setup.py
#  libmsym
#
#  Created by Marcus Johansson on 07/10/15.
#  Copyright (c) 2015 Marcus Johansson.
#
#  Distributed under the MIT License ( See LICENSE file or copy at http://opensource.org/licenses/MIT )
#

from distutils.core import setup
import sys

if (not sys.version_info[0] is 3):
    sys.exit('libmsym module requires python 3')
    
setup(name='libmsym',
      version='0.2.4',
      description = 'libmsym python binding',
      license='MIT',
      author='Marcus Johansson',
      author_email='mcodev31@gmail.com',
      url='https://github.com/mcodev31/libmsym',
      packages=['libmsym']
      )



#!/usr/bin/env python

import unittest

import gi
gi.require_version('v_sim', '3.8')
from gi.repository import GLib, v_sim

import signals

class TestNodeArray(unittest.TestCase):
    def setUp(self):
        super(TestNodeArray, self).setUp()
        self.addTypeEqualityFunc(list, self.fuzzyList)
        self.addTypeEqualityFunc(float, self.fuzzyFloat)

    def fuzzyList(self, a, b, msg = None):
        if len(a) > 0 and isinstance(a[0], float) and len(a) == len(b):
            try:
                for i, j in zip(a, b):
                    self.fuzzyFloat(i, j, msg)
            except (self.failureException):
              raise self.failureException(msg if msg is not None else "" + "delta %s" % ([abs(i - j) for i, j in zip(a, b)]))
            return
        return self.assertListEqual(a, b, msg)
        
    def fuzzyFloat(self, a, b, msg = None):
        if abs(b-a) > 6e-8:
            raise self.failureException(msg)
        
    def test_population(self):
        # Empty structure.
        array = v_sim.Data.new()
        self.assertEqual(array.getNElements(True), 0)
        self.assertEqual(array.getNElements(False), 0)
        self.assertEqual(array.getNNodes(), 0)

        # Population allocation
        array.allocateByNames((1, 2, 45), ("O", "H", "g"))
        self.assertTrue(v_sim.Element.lookup("O") is not None)
        self.assertTrue(v_sim.Element.lookup("H") is not None)
        self.assertTrue(v_sim.Element.lookup("g") is not None)
        self.assertEqual(array.getNElements(True), 0)
        self.assertEqual(array.getNElements(False), 0)
        self.assertEqual(array.getNNodes(), 0)
        self.assertTrue(array.containsElement(v_sim.Element.lookup("O")))
        self.assertTrue(array.containsElement(v_sim.Element.lookup("H")))
        self.assertTrue(array.containsElement(v_sim.Element.lookup("g")))
        self.assertEqual(array.getElementId(v_sim.Element.lookup("O")), 0)
        self.assertEqual(array.getElementId(v_sim.Element.lookup("H")), 1)
        self.assertEqual(array.getElementId(v_sim.Element.lookup("g")), 2)

        # Population partial re-allocation
        array.allocateByNames((1, 4), ("C", "H"))
        self.assertTrue(v_sim.Element.lookup("C") is not None)
        self.assertEqual(array.getNElements(True), 0)
        self.assertEqual(array.getNElements(False), 0)
        self.assertEqual(array.getNNodes(), 0)
        self.assertTrue(array.containsElement(v_sim.Element.lookup("O")))
        self.assertTrue(array.containsElement(v_sim.Element.lookup("H")))
        self.assertTrue(array.containsElement(v_sim.Element.lookup("g")))
        self.assertTrue(array.containsElement(v_sim.Element.lookup("C")))
        self.assertEqual(array.getElementId(v_sim.Element.lookup("O")), 0)
        self.assertEqual(array.getElementId(v_sim.Element.lookup("H")), 1)
        self.assertEqual(array.getElementId(v_sim.Element.lookup("g")), 2)
        self.assertEqual(array.getElementId(v_sim.Element.lookup("C")), 3)

        # Node adding
        node = array.getNewNode(v_sim.Element.lookup("O"))
        self.assertTrue(node is not None)
        self.assertEqual(node.xyz, [0., 0., 0.])
        self.assertEqual(node.translation, [0., 0., 0.])
        self.assertTrue(not node.getVisibility())
        node.newValues((1,2,3))
        self.assertEqual(node.number, 0)
        self.assertEqual(node.xyz, [1., 2., 3.])
        self.assertEqual(node.translation, [0., 0., 0.])
        self.assertTrue(node.getVisibility())
        self.assertEqual(array.getNElements(True), 1)
        self.assertEqual(array.getNElements(False), 1)
        self.assertEqual(array.getNNodes(), 1)
        self.assertEqual(array.getNOriginalNodes(), 1)
        self.assertTrue(array.getElement(node), v_sim.Element.lookup("O"))
        self.assertEqual(array.getFromId(0), node)
        self.assertTrue(array.getFromId(1) is None)
        self.assertEqual(array.getOriginal(0), -1)
        self.assertEqual(array.get_property("n-nodes"), 1)

        node = array.getNewNodeForId(1)
        self.assertTrue(node is not None)
        node.newValues((2,0,0))
        self.assertEqual(node.number, 1)
        self.assertEqual(node.xyz, [2., 0., 0.])
        self.assertEqual(node.translation, [0., 0., 0.])
        self.assertTrue(node.getVisibility())
        self.assertEqual(array.getNElements(True), 2)
        self.assertEqual(array.getNElements(False), 2)
        self.assertEqual(array.getNNodes(), 2)
        self.assertEqual(array.getNOriginalNodes(), 2)
        self.assertTrue(array.getElement(node), v_sim.Element.lookup("H"))
        self.assertEqual(array.getFromId(1), node)
        self.assertTrue(array.getFromId(0) is not None)
        self.assertTrue(array.getFromId(2) is None)
        self.assertEqual(array.getOriginal(1), -1)

        node = array.copyNode(node)
        self.assertTrue(node is not None)
        node.newValues((-2,0,0))
        self.assertEqual(node.number, 2)
        self.assertEqual(node.xyz, [-2., 0., 0.])
        self.assertEqual(node.translation, [0., 0., 0.])
        self.assertTrue(node.getVisibility())
        self.assertEqual(array.getNElements(True), 2)
        self.assertEqual(array.getNElements(False), 2)
        self.assertEqual(array.getNNodes(), 3)
        self.assertEqual(array.getNOriginalNodes(), 2)
        self.assertTrue(array.getElement(node), v_sim.Element.lookup("H"))
        self.assertEqual(array.getFromId(2), node)
        self.assertTrue(array.getFromId(1) is not None)
        self.assertTrue(array.getFromId(3) is None)
        self.assertEqual(array.getOriginal(2), 1)
        self.assertTrue(array.setOriginal(2))
        self.assertEqual(array.getOriginal(2), -1)
        self.assertEqual(array.getNOriginalNodes(), 3)

        # Node removal
        array.removeNodes((0, 1))
        self.assertEqual(array.getNElements(True), 1)
        self.assertEqual(array.getNNodes(), 1)
        self.assertTrue(array.getFromId(0) is None)
        self.assertTrue(array.getFromId(1) is None)
        self.assertTrue(array.getFromId(2) is not None)

        node = array.copyNode(node)
        self.assertTrue(array.getFromId(3) is not None)
        array.removeAllDuplicateNodes()
        self.assertTrue(array.getFromId(3) is None)
        self.assertEqual(array.getNElements(True), 1)
        self.assertEqual(array.getNNodes(), 1)

        array.removeNodesOfElement(v_sim.Element.lookup("H"))
        self.assertEqual(array.getNElements(True), 0)
        self.assertEqual(array.getNNodes(), 0)

    def test_populationSignals(self):
      array = v_sim.Data.new()
      with signals.Listener(array, "notify::n-nodes") as nodes:
        with signals.Listener(array, "notify::elements") as elems:
          with signals.Listener(array, "PopulationIncrease") as popInc:
            with signals.Listener(array, "PopulationDecrease") as popDec:
              self.assertEqual(nodes.triggered(), 0)
              self.assertEqual(elems.triggered(), 0)
              self.assertEqual(popInc.triggered(), 0)
              self.assertEqual(popDec.triggered(), 0)
      
              array.allocateByNames((1, 4), ("C", "H"))
              self.assertEqual(nodes.triggered(), 0)
              self.assertEqual(elems.triggered(), 1)
              self.assertEqual(popInc.triggered(), 0)
              self.assertEqual(popDec.triggered(), 0)
              elems.reset()

              array.getNewNodeForId(0)
              self.assertEqual(nodes.triggered(), 1)
              self.assertEqual(elems.triggered(), 0)
              self.assertEqual(popInc.triggered(), 1)
              self.assertEqual(popDec.triggered(), 0)
              nodes.reset()
              popInc.reset()

              array.startAdding()
              array.getNewNodeForId(1)
              array.getNewNodeForId(1)
              array.getNewNodeForId(1)
              array.getNewNodeForId(1)
              self.assertEqual(nodes.triggered(), 0)
              self.assertEqual(popInc.triggered(), 0)
              self.assertEqual(popDec.triggered(), 0)
              array.completeAdding()
              self.assertEqual(nodes.triggered(), 1)
              self.assertEqual(popInc.triggered(), 1)
              self.assertEqual(popDec.triggered(), 0)
              nodes.reset()
              popInc.reset()

              array.removeNodes((1, 2))
              self.assertEqual(nodes.triggered(), 1)
              self.assertEqual(popInc.triggered(), 0)
              self.assertEqual(popDec.triggered(), 1)

              array.removeNodesOfElement(v_sim.Element.lookup("C"))
              self.assertEqual(nodes.triggered(), 2)
              self.assertEqual(popInc.triggered(), 0)
              self.assertEqual(popDec.triggered(), 2)

              array.freeNodes()
              self.assertEqual(nodes.triggered(), 3)
              self.assertEqual(elems.triggered(), 1)
              self.assertEqual(popInc.triggered(), 0)
              self.assertEqual(popDec.triggered(), 2) # freeNodes() does not emit PopulationDecrease

    def test_nodeMovements(self):
      array = v_sim.Data.new()
      array.allocateByNames((1, 4), ("C", "H"))
      array.getNewNodeForId(0).newValues(( 0, 0, 0))
      array.getNewNodeForId(1).newValues((-1,-1,-1))
      array.getNewNodeForId(1).newValues(( 1, 1,-1))
      array.getNewNodeForId(1).newValues(( 1,-1, 1))
      array.getNewNodeForId(1).newValues((-1, 1, 1))

      with signals.Listener(array, "position-changed") as moves:
        array.getFromId(0).setCoordinates((2.,2.,2.))
        self.assertEqual(array.getFromId(0).xyz, [2.,2.,2.])
        self.assertEqual(moves.triggered(), 0)

        array.moveNode(0, (0.,0.,0.))
        self.assertEqual(array.getFromId(0).xyz, [0.,0.,0.])
        self.assertEqual(moves.triggered(), 1)
        array.moveNodes((3,4), (4.,5.,6.,7.,8.,9.))
        self.assertEqual(array.getFromId(3).xyz, [4.,5.,6.])
        self.assertEqual(array.getFromId(4).xyz, [7.,8.,9.])
        self.assertEqual(moves.triggered(), 2)
        moves.reset()

        array.shiftNode(0, (1,2,3))
        self.assertEqual(array.getFromId(0).xyz, [1,2,3])
        self.assertEqual(moves.triggered(), 1)
        array.shiftNodes((3,4), (-3.,-2.,-1.))
        self.assertEqual(array.getFromId(3).xyz, [1.,3.,5.])
        self.assertEqual(array.getFromId(4).xyz, [4.,6.,8.])
        self.assertEqual(moves.triggered(), 2)
        moves.reset()

        array.startMoving()
        array.shiftNode(0, (-1,-2,-3))
        array.moveNodes((3, 4), (1,-1,1,-1,1,1))
        array.rotateNodes((1,2), (0,0,1), (0,0,0), 45);
        self.assertEqual(moves.triggered(), 0)
        array.completeMoving()
        self.assertEqual(moves.triggered(), 1)
        moves.reset()

        array.rotateNodes((1,2), (0,0,1), (0,0,0), 45);
        self.assertEqual(array.getFromId(1).xyz, [1.,-1.,-1.])
        self.assertEqual(array.getFromId(2).xyz, [-1.,1.,-1.])
        self.assertEqual(moves.triggered(), 1)

if __name__ == '__main__':
    unittest.main()
